@@ -25,7 +25,7 @@
 #include "main/mtypes.h"
 #include "main/macros.h"
 #include "util/ralloc.h"
-#include "util/string_to_uint_map.h"
+#include "string_to_uint_map.h"
 #include "uniform_initializer_utils.h"
 
 namespace linker {
@@ -68,6 +68,8 @@ public:
 void
 set_uniform_initializer::SetUp()
 {
+   glsl_type_singleton_init_or_ref();
+
    this->mem_ctx = ralloc_context(NULL);
    this->prog = rzalloc(NULL, struct gl_shader_program);
    this->prog->data = rzalloc(this->prog, struct gl_shader_program_data);
@@ -86,6 +88,8 @@ set_uniform_initializer::TearDown()
 
    ralloc_free(this->prog);
    this->prog = NULL;
+
+   glsl_type_singleton_decref();
 }
 
 /**
