@@ -108,6 +108,7 @@ void
 nir_shader_add_variable(nir_shader *shader, nir_variable *var)
 {
    switch (var->data.mode) {
+   case nir_num_variable_modes:
    case nir_var_all:
       assert(!"invalid mode");
       break;
@@ -146,6 +147,10 @@ nir_shader_add_variable(nir_shader *shader, nir_variable *var)
    case nir_var_system_value:
       exec_list_push_tail(&shader->system_values, &var->node);
       break;
+
+   case nir_var_mem_push_const:
+      assert(!"nir_var_push_constant is not supposed to be used for variables");
+      break;
    }
 }
 
@@ -1416,7 +1421,7 @@ nir_instr_rewrite_dest(nir_instr *instr, nir_dest *dest, nir_dest new_dest)
 {
    if (dest->is_ssa) {
       /* We can only overwrite an SSA destination if it has no uses. */
-      assert(list_empty(&dest->ssa.uses) && list_empty(&dest->ssa.if_uses));
+      assert(list_is_empty(&dest->ssa.uses) && list_is_empty(&dest->ssa.if_uses));
    } else {
       list_del(&dest->reg.def_link);
       if (dest->reg.indirect)
@@ -1547,7 +1552,7 @@ nir_ssa_def_components_read(const nir_ssa_def *def)
       }
    }
 
-   if (!list_empty(&def->if_uses))
+   if (!list_is_empty(&def->if_uses))
       read_mask |= 1;
 
    return read_mask;
@@ -1787,6 +1792,39 @@ nir_index_instrs(nir_function_impl *impl)
    return index;
 }
 
+static void
+index_var_list(struct exec_list *list)
+{
+   unsigned next_index = 0;
+   nir_foreach_variable(var, list)
+      var->index = next_index++;
+}
+
+void
+nir_index_vars(nir_shader *shader, nir_function_impl *impl, nir_variable_mode modes)
+{
+   if ((modes & nir_var_function_temp) && impl)
+      index_var_list(&impl->locals);
+
+   if (modes & nir_var_shader_temp)
+      index_var_list(&shader->globals);
+
+   if (modes & nir_var_shader_in)
+      index_var_list(&shader->inputs);
+
+   if (modes & nir_var_shader_out)
+      index_var_list(&shader->outputs);
+
+   if (modes & (nir_var_uniform | nir_var_mem_ubo | nir_var_mem_ssbo))
+      index_var_list(&shader->uniforms);
+
+   if (modes & nir_var_mem_shared)
+      index_var_list(&shader->shared);
+
+   if (modes & nir_var_system_value)
+      index_var_list(&shader->system_values);
+}
+
 static nir_instr *
 cursor_next_instr(nir_cursor cursor)
 {
@@ -1823,9 +1861,10 @@ cursor_next_instr(nir_cursor cursor)
    unreachable("Inavlid cursor option");
 }
 
-static bool
+ASSERTED static bool
 dest_is_ssa(nir_dest *dest, void *_state)
 {
+   (void) _state;
    return dest->is_ssa;
 }
 
@@ -1888,7 +1927,7 @@ nir_function_impl_lower_instructions(nir_function_impl *impl,
          list_for_each_entry_safe(nir_src, use_src, &old_if_uses, use_link)
             nir_if_rewrite_condition(use_src->parent_if, new_src);
 
-         if (list_empty(&old_def->uses) && list_empty(&old_def->if_uses)) {
+         if (list_is_empty(&old_def->uses) && list_is_empty(&old_def->if_uses)) {
             iter = nir_instr_remove(instr);
          } else {
             iter = nir_after_instr(instr);
@@ -2185,8 +2224,10 @@ nir_rewrite_image_intrinsic(nir_intrinsic_instr *intrin, nir_ssa_def *src,
    CASE(load)
    CASE(store)
    CASE(atomic_add)
-   CASE(atomic_min)
-   CASE(atomic_max)
+   CASE(atomic_imin)
+   CASE(atomic_umin)
+   CASE(atomic_imax)
+   CASE(atomic_umax)
    CASE(atomic_and)
    CASE(atomic_or)
    CASE(atomic_xor)
@@ -2207,7 +2248,7 @@ nir_rewrite_image_intrinsic(nir_intrinsic_instr *intrin, nir_ssa_def *src,
 
    nir_intrinsic_set_image_dim(intrin, glsl_get_sampler_dim(deref->type));
    nir_intrinsic_set_image_array(intrin, glsl_sampler_type_is_array(deref->type));
-   nir_intrinsic_set_access(intrin, access | var->data.image.access);
+   nir_intrinsic_set_access(intrin, access | var->data.access);
    nir_intrinsic_set_format(intrin, var->data.image.format);
 
    nir_instr_rewrite_src(&intrin->instr, &intrin->src[0],
