@@ -51,8 +51,41 @@ unpack_data(const void *p)
    return (struct ssa_result_range){v & 0xff, (v & 0x0ff00) != 0};
 }
 
+static void *
+pack_key(const struct nir_alu_instr *instr, nir_alu_type type)
+{
+   uintptr_t type_encoding;
+   uintptr_t ptr = (uintptr_t) instr;
+
+   /* The low 2 bits have to be zero or this whole scheme falls apart. */
+   assert((ptr & 0x3) == 0);
+
+   /* NIR is typeless in the sense that sequences of bits have whatever
+    * meaning is attached to them by the instruction that consumes them.
+    * However, the number of bits must match between producer and consumer.
+    * As a result, the number of bits does not need to be encoded here.
+    */
+   switch (nir_alu_type_get_base_type(type)) {
+   case nir_type_int:   type_encoding = 0; break;
+   case nir_type_uint:  type_encoding = 1; break;
+   case nir_type_bool:  type_encoding = 2; break;
+   case nir_type_float: type_encoding = 3; break;
+   default: unreachable("Invalid base type.");
+   }
+
+   return (void *)(ptr | type_encoding);
+}
+
+static nir_alu_type
+nir_alu_src_type(const nir_alu_instr *instr, unsigned src)
+{
+   return nir_alu_type_get_base_type(nir_op_infos[instr->op].input_types[src]) |
+          nir_src_bit_size(instr->src[src].src);
+}
+
 static struct ssa_result_range
-analyze_constant(const struct nir_alu_instr *instr, unsigned src)
+analyze_constant(const struct nir_alu_instr *instr, unsigned src,
+                 nir_alu_type use_type)
 {
    uint8_t swizzle[4] = { 0, 1, 2, 3 };
 
@@ -69,7 +102,7 @@ analyze_constant(const struct nir_alu_instr *instr, unsigned src)
 
    struct ssa_result_range r = { unknown, false };
 
-   switch (nir_op_infos[instr->op].input_types[src]) {
+   switch (nir_alu_type_get_base_type(use_type)) {
    case nir_type_float: {
       double min_value = DBL_MAX;
       double max_value = -DBL_MAX;
@@ -179,31 +212,171 @@ analyze_constant(const struct nir_alu_instr *instr, unsigned src)
    }
 }
 
-#ifndef NDEBUG
-#define ASSERT_TABLE_IS_COMMUTATIVE(t)                        \
-   do {                                                       \
-      for (unsigned r = 0; r < ARRAY_SIZE(t); r++) {          \
-         for (unsigned c = 0; c < ARRAY_SIZE(t[0]); c++)      \
-            assert(t[r][c] == t[c][r]);                       \
-      }                                                       \
-   } while (false)
-
-#define ASSERT_TABLE_IS_DIAGONAL(t)                           \
-   do {                                                       \
-      for (unsigned r = 0; r < ARRAY_SIZE(t); r++)            \
-         assert(t[r][r] == r);                                \
-   } while (false)
-#else
-#define ASSERT_TABLE_IS_COMMUTATIVE(t)
-#define ASSERT_TABLE_IS_DIAGONAL(t)
-#endif
-
 /**
  * Short-hand name for use in the tables in analyze_expression.  If this name
  * becomes a problem on some compiler, we can change it to _.
  */
 #define _______ unknown
 
+
+/* MSVC doesn't have C99's _Pragma() */
+#ifdef _MSC_VER
+#define _Pragma(x)
+#endif
+
+
+#ifndef NDEBUG
+#define ASSERT_TABLE_IS_COMMUTATIVE(t)                        \
+   do {                                                       \
+      static bool first = true;                               \
+      if (first) {                                            \
+         first = false;                                       \
+         _Pragma("GCC unroll 7")                              \
+         for (unsigned r = 0; r < ARRAY_SIZE(t); r++) {       \
+            _Pragma("GCC unroll 7")                           \
+            for (unsigned c = 0; c < ARRAY_SIZE(t[0]); c++)   \
+               assert(t[r][c] == t[c][r]);                    \
+         }                                                    \
+      }                                                       \
+   } while (false)
+
+#define ASSERT_TABLE_IS_DIAGONAL(t)                           \
+   do {                                                       \
+      static bool first = true;                               \
+      if (first) {                                            \
+         first = false;                                       \
+         _Pragma("GCC unroll 7")                              \
+         for (unsigned r = 0; r < ARRAY_SIZE(t); r++)         \
+            assert(t[r][r] == r);                             \
+      }                                                       \
+   } while (false)
+
+static enum ssa_ranges
+union_ranges(enum ssa_ranges a, enum ssa_ranges b)
+{
+   static const enum ssa_ranges union_table[last_range + 1][last_range + 1] = {
+      /* left\right   unknown  lt_zero  le_zero  gt_zero  ge_zero  ne_zero  eq_zero */
+      /* unknown */ { _______, _______, _______, _______, _______, _______, _______ },
+      /* lt_zero */ { _______, lt_zero, le_zero, ne_zero, _______, ne_zero, le_zero },
+      /* le_zero */ { _______, le_zero, le_zero, _______, _______, _______, le_zero },
+      /* gt_zero */ { _______, ne_zero, _______, gt_zero, ge_zero, ne_zero, ge_zero },
+      /* ge_zero */ { _______, _______, _______, ge_zero, ge_zero, _______, ge_zero },
+      /* ne_zero */ { _______, ne_zero, _______, ne_zero, _______, ne_zero, _______ },
+      /* eq_zero */ { _______, le_zero, le_zero, ge_zero, ge_zero, _______, eq_zero },
+   };
+
+   ASSERT_TABLE_IS_COMMUTATIVE(union_table);
+   ASSERT_TABLE_IS_DIAGONAL(union_table);
+
+   return union_table[a][b];
+}
+
+/* Verify that the 'unknown' entry in each row (or column) of the table is the
+ * union of all the other values in the row (or column).
+ */
+#define ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_2_SOURCE(t)              \
+   do {                                                                 \
+      static bool first = true;                                         \
+      if (first) {                                                      \
+         first = false;                                                 \
+         _Pragma("GCC unroll 7")                                        \
+         for (unsigned i = 0; i < last_range; i++) {                    \
+            enum ssa_ranges col_range = t[i][unknown + 1];              \
+            enum ssa_ranges row_range = t[unknown + 1][i];              \
+                                                                        \
+            _Pragma("GCC unroll 5")                                     \
+            for (unsigned j = unknown + 2; j < last_range; j++) {       \
+               col_range = union_ranges(col_range, t[i][j]);            \
+               row_range = union_ranges(row_range, t[j][i]);            \
+            }                                                           \
+                                                                        \
+            assert(col_range == t[i][unknown]);                         \
+            assert(row_range == t[unknown][i]);                         \
+         }                                                              \
+      }                                                                 \
+   } while (false)
+
+/* For most operations, the union of ranges for a strict inequality and
+ * equality should be the range of the non-strict inequality (e.g.,
+ * union_ranges(range(op(lt_zero), range(op(eq_zero))) == range(op(le_zero)).
+ *
+ * Does not apply to selection-like opcodes (bcsel, fmin, fmax, etc.).
+ */
+#define ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_1_SOURCE(t) \
+   do {                                                                 \
+      assert(union_ranges(t[lt_zero], t[eq_zero]) == t[le_zero]);       \
+      assert(union_ranges(t[gt_zero], t[eq_zero]) == t[ge_zero]);       \
+   } while (false)
+
+#define ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_2_SOURCE(t) \
+   do {                                                                 \
+      static bool first = true;                                         \
+      if (first) {                                                      \
+         first = false;                                                 \
+         _Pragma("GCC unroll 7")                                        \
+         for (unsigned i = 0; i < last_range; i++) {                    \
+            assert(union_ranges(t[i][lt_zero], t[i][eq_zero]) == t[i][le_zero]); \
+            assert(union_ranges(t[i][gt_zero], t[i][eq_zero]) == t[i][ge_zero]); \
+            assert(union_ranges(t[lt_zero][i], t[eq_zero][i]) == t[le_zero][i]); \
+            assert(union_ranges(t[gt_zero][i], t[eq_zero][i]) == t[ge_zero][i]); \
+         }                                                              \
+      }                                                                 \
+   } while (false)
+
+/* Several other unordered tuples span the range of "everything."  Each should
+ * have the same value as unknown: (lt_zero, ge_zero), (le_zero, gt_zero), and
+ * (eq_zero, ne_zero).  union_ranges is already commutative, so only one
+ * ordering needs to be checked.
+ *
+ * Does not apply to selection-like opcodes (bcsel, fmin, fmax, etc.).
+ *
+ * In cases where this can be used, it is unnecessary to also use
+ * ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_*_SOURCE.  For any range X,
+ * union_ranges(X, X) == X.  The disjoint ranges cover all of the non-unknown
+ * possibilities, so the union of all the unions of disjoint ranges is
+ * equivalent to the union of "others."
+ */
+#define ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_1_SOURCE(t)            \
+   do {                                                                 \
+      assert(union_ranges(t[lt_zero], t[ge_zero]) == t[unknown]);       \
+      assert(union_ranges(t[le_zero], t[gt_zero]) == t[unknown]);       \
+      assert(union_ranges(t[eq_zero], t[ne_zero]) == t[unknown]);       \
+   } while (false)
+
+#define ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_2_SOURCE(t)            \
+   do {                                                                 \
+      static bool first = true;                                         \
+      if (first) {                                                      \
+         first = false;                                                 \
+         _Pragma("GCC unroll 7")                                        \
+         for (unsigned i = 0; i < last_range; i++) {                    \
+            assert(union_ranges(t[i][lt_zero], t[i][ge_zero]) ==        \
+                   t[i][unknown]);                                      \
+            assert(union_ranges(t[i][le_zero], t[i][gt_zero]) ==        \
+                   t[i][unknown]);                                      \
+            assert(union_ranges(t[i][eq_zero], t[i][ne_zero]) ==        \
+                   t[i][unknown]);                                      \
+                                                                        \
+            assert(union_ranges(t[lt_zero][i], t[ge_zero][i]) ==        \
+                   t[unknown][i]);                                      \
+            assert(union_ranges(t[le_zero][i], t[gt_zero][i]) ==        \
+                   t[unknown][i]);                                      \
+            assert(union_ranges(t[eq_zero][i], t[ne_zero][i]) ==        \
+                   t[unknown][i]);                                      \
+         }                                                              \
+      }                                                                 \
+   } while (false)
+
+#else
+#define ASSERT_TABLE_IS_COMMUTATIVE(t)
+#define ASSERT_TABLE_IS_DIAGONAL(t)
+#define ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_2_SOURCE(t)
+#define ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_1_SOURCE(t)
+#define ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_2_SOURCE(t)
+#define ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_1_SOURCE(t)
+#define ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_2_SOURCE(t)
+#endif
+
 /**
  * Analyze an expression to determine the range of its result
  *
@@ -215,13 +388,16 @@ analyze_constant(const struct nir_alu_instr *instr, unsigned src)
  */
 static struct ssa_result_range
 analyze_expression(const nir_alu_instr *instr, unsigned src,
-                   struct hash_table *ht)
+                   struct hash_table *ht, nir_alu_type use_type)
 {
+   /* Ensure that the _Pragma("GCC unroll 7") above are correct. */
+   STATIC_ASSERT(last_range + 1 == 7);
+
    if (!instr->src[src].src.is_ssa)
       return (struct ssa_result_range){unknown, false};
 
    if (nir_src_is_const(instr->src[src].src))
-      return analyze_constant(instr, src);
+      return analyze_constant(instr, src, use_type);
 
    if (instr->src[src].src.ssa->parent_instr->type != nir_instr_type_alu)
       return (struct ssa_result_range){unknown, false};
@@ -229,8 +405,6 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    const struct nir_alu_instr *const alu =
        nir_instr_as_alu(instr->src[src].src.ssa->parent_instr);
 
-   const nir_alu_type use_type = nir_op_infos[instr->op].input_types[src];
-
    /* Bail if the type of the instruction generating the value does not match
     * the type the value will be interpreted as.  int/uint/bool can be
     * reinterpreted trivially.  The most important cases are between float and
@@ -249,7 +423,7 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       }
    }
 
-   struct hash_entry *he = _mesa_hash_table_search(ht, alu);
+   struct hash_entry *he = _mesa_hash_table_search(ht, pack_key(alu, use_type));
    if (he != NULL)
       return unpack_data(he->data);
 
@@ -295,6 +469,8 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    };
 
    ASSERT_TABLE_IS_COMMUTATIVE(fadd_table);
+   ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_2_SOURCE(fadd_table);
+   ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_2_SOURCE(fadd_table);
 
    /* Due to flush-to-zero semanatics of floating-point numbers with very
     * small mangnitudes, we can never really be sure a result will be
@@ -339,12 +515,17 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    };
 
    ASSERT_TABLE_IS_COMMUTATIVE(fmul_table);
+   ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_2_SOURCE(fmul_table);
+   ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_2_SOURCE(fmul_table);
 
    static const enum ssa_ranges fneg_table[last_range + 1] = {
    /* unknown  lt_zero  le_zero  gt_zero  ge_zero  ne_zero  eq_zero */
       _______, gt_zero, ge_zero, lt_zero, le_zero, ne_zero, eq_zero
    };
 
+   ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_1_SOURCE(fneg_table);
+   ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_1_SOURCE(fneg_table);
+
 
    switch (alu->op) {
    case nir_op_b2f32:
@@ -353,21 +534,10 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       break;
 
    case nir_op_bcsel: {
-      const struct ssa_result_range left = analyze_expression(alu, 1, ht);
-      const struct ssa_result_range right = analyze_expression(alu, 2, ht);
-
-      /* If either source is a constant load that is not zero, punt.  The type
-       * will always be uint regardless of the actual type.  We can't even
-       * decide if the value is non-zero because -0.0 is 0x80000000, and that
-       * will (possibly incorrectly) be considered non-zero.
-       */
-      /* FINISHME: We could do better, but it would require having the expected
-       * FINISHME: type passed in.
-       */
-      if ((nir_src_is_const(alu->src[1].src) && left.range != eq_zero) ||
-          (nir_src_is_const(alu->src[2].src) && right.range != eq_zero)) {
-         return (struct ssa_result_range){unknown, false};
-      }
+      const struct ssa_result_range left =
+         analyze_expression(alu, 1, ht, use_type);
+      const struct ssa_result_range right =
+         analyze_expression(alu, 2, ht, use_type);
 
       r.is_integral = left.is_integral && right.is_integral;
 
@@ -424,6 +594,7 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
 
       ASSERT_TABLE_IS_COMMUTATIVE(table);
       ASSERT_TABLE_IS_DIAGONAL(table);
+      ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_2_SOURCE(table);
 
       r.range = table[left.range][right.range];
       break;
@@ -431,7 +602,7 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
 
    case nir_op_i2f32:
    case nir_op_u2f32:
-      r = analyze_expression(alu, 0, ht);
+      r = analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       r.is_integral = true;
 
@@ -441,7 +612,7 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       break;
 
    case nir_op_fabs:
-      r = analyze_expression(alu, 0, ht);
+      r = analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       switch (r.range) {
       case unknown:
@@ -463,8 +634,10 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       break;
 
    case nir_op_fadd: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range right = analyze_expression(alu, 1, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range right =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
 
       r.is_integral = left.is_integral && right.is_integral;
       r.range = fadd_table[left.range][right.range];
@@ -481,7 +654,10 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
          ge_zero, ge_zero, ge_zero, gt_zero, gt_zero, ge_zero, gt_zero
       };
 
-      r = analyze_expression(alu, 0, ht);
+      r = analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+
+      ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_1_SOURCE(table);
+      ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_1_SOURCE(table);
 
       r.is_integral = r.is_integral && is_not_negative(r.range);
       r.range = table[r.range];
@@ -489,8 +665,10 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    }
 
    case nir_op_fmax: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range right = analyze_expression(alu, 1, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range right =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
 
       r.is_integral = left.is_integral && right.is_integral;
 
@@ -545,14 +723,17 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       /* Treat fmax as commutative. */
       ASSERT_TABLE_IS_COMMUTATIVE(table);
       ASSERT_TABLE_IS_DIAGONAL(table);
+      ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_2_SOURCE(table);
 
       r.range = table[left.range][right.range];
       break;
    }
 
    case nir_op_fmin: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range right = analyze_expression(alu, 1, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range right =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
 
       r.is_integral = left.is_integral && right.is_integral;
 
@@ -607,14 +788,17 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       /* Treat fmin as commutative. */
       ASSERT_TABLE_IS_COMMUTATIVE(table);
       ASSERT_TABLE_IS_DIAGONAL(table);
+      ASSERT_UNION_OF_OTHERS_MATCHES_UNKNOWN_2_SOURCE(table);
 
       r.range = table[left.range][right.range];
       break;
    }
 
    case nir_op_fmul: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range right = analyze_expression(alu, 1, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range right =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
 
       r.is_integral = left.is_integral && right.is_integral;
 
@@ -634,28 +818,24 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    }
 
    case nir_op_frcp:
-      r = (struct ssa_result_range){analyze_expression(alu, 0, ht).range, false};
+      r = (struct ssa_result_range){
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0)).range,
+         false
+      };
       break;
 
-   case nir_op_mov: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
-
-      /* See commentary in nir_op_bcsel for the reasons this is necessary. */
-      if (nir_src_is_const(alu->src[0].src) && left.range != eq_zero)
-         return (struct ssa_result_range){unknown, false};
-
-      r = left;
+   case nir_op_mov:
+      r = analyze_expression(alu, 0, ht, use_type);
       break;
-   }
 
    case nir_op_fneg:
-      r = analyze_expression(alu, 0, ht);
+      r = analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       r.range = fneg_table[r.range];
       break;
 
    case nir_op_fsat:
-      r = analyze_expression(alu, 0, ht);
+      r = analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       switch (r.range) {
       case le_zero:
@@ -680,7 +860,10 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       break;
 
    case nir_op_fsign:
-      r = (struct ssa_result_range){analyze_expression(alu, 0, ht).range, true};
+      r = (struct ssa_result_range){
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0)).range,
+         true
+      };
       break;
 
    case nir_op_fsqrt:
@@ -689,7 +872,8 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       break;
 
    case nir_op_ffloor: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       r.is_integral = true;
 
@@ -704,7 +888,8 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    }
 
    case nir_op_fceil: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       r.is_integral = true;
 
@@ -719,7 +904,8 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    }
 
    case nir_op_ftrunc: {
-      const struct ssa_result_range left = analyze_expression(alu, 0, ht);
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
 
       r.is_integral = true;
 
@@ -749,10 +935,78 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
       r = (struct ssa_result_range){le_zero, false};
       break;
 
+   case nir_op_fpow: {
+      /* Due to flush-to-zero semanatics of floating-point numbers with very
+       * small mangnitudes, we can never really be sure a result will be
+       * non-zero.
+       *
+       * NIR uses pow() and powf() to constant evaluate nir_op_fpow.  The man
+       * page for that function says:
+       *
+       *    If y is 0, the result is 1.0 (even if x is a NaN).
+       *
+       * gt_zero: pow(*, eq_zero)
+       *        | pow(eq_zero, lt_zero)   # 0^-y = +inf
+       *        | pow(eq_zero, le_zero)   # 0^-y = +inf or 0^0 = 1.0
+       *        ;
+       *
+       * eq_zero: pow(eq_zero, gt_zero)
+       *        ;
+       *
+       * ge_zero: pow(gt_zero, gt_zero)
+       *        | pow(gt_zero, ge_zero)
+       *        | pow(gt_zero, lt_zero)
+       *        | pow(gt_zero, le_zero)
+       *        | pow(gt_zero, ne_zero)
+       *        | pow(gt_zero, unknown)
+       *        | pow(ge_zero, gt_zero)
+       *        | pow(ge_zero, ge_zero)
+       *        | pow(ge_zero, lt_zero)
+       *        | pow(ge_zero, le_zero)
+       *        | pow(ge_zero, ne_zero)
+       *        | pow(ge_zero, unknown)
+       *        | pow(eq_zero, ge_zero)  # 0^0 = 1.0 or 0^+y = 0.0
+       *        | pow(eq_zero, ne_zero)  # 0^-y = +inf or 0^+y = 0.0
+       *        | pow(eq_zero, unknown)  # union of all other y cases
+       *        ;
+       *
+       * All other cases are unknown.
+       *
+       * We could do better if the right operand is a constant, integral
+       * value.
+       */
+      static const enum ssa_ranges table[last_range + 1][last_range + 1] = {
+         /* left\right   unknown  lt_zero  le_zero  gt_zero  ge_zero  ne_zero  eq_zero */
+         /* unknown */ { _______, _______, _______, _______, _______, _______, gt_zero },
+         /* lt_zero */ { _______, _______, _______, _______, _______, _______, gt_zero },
+         /* le_zero */ { _______, _______, _______, _______, _______, _______, gt_zero },
+         /* gt_zero */ { ge_zero, ge_zero, ge_zero, ge_zero, ge_zero, ge_zero, gt_zero },
+         /* ge_zero */ { ge_zero, ge_zero, ge_zero, ge_zero, ge_zero, ge_zero, gt_zero },
+         /* ne_zero */ { _______, _______, _______, _______, _______, _______, gt_zero },
+         /* eq_zero */ { ge_zero, gt_zero, gt_zero, eq_zero, ge_zero, ge_zero, gt_zero },
+      };
+
+      const struct ssa_result_range left =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range right =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
+
+      ASSERT_UNION_OF_DISJOINT_MATCHES_UNKNOWN_2_SOURCE(table);
+      ASSERT_UNION_OF_EQ_AND_STRICT_INEQ_MATCHES_NONSTRICT_2_SOURCE(table);
+
+      r.is_integral = left.is_integral && right.is_integral &&
+                      is_not_negative(right.range);
+      r.range = table[left.range][right.range];
+      break;
+   }
+
    case nir_op_ffma: {
-      const struct ssa_result_range first = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range second = analyze_expression(alu, 1, ht);
-      const struct ssa_result_range third = analyze_expression(alu, 2, ht);
+      const struct ssa_result_range first =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range second =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
+      const struct ssa_result_range third =
+         analyze_expression(alu, 2, ht, nir_alu_src_type(alu, 2));
 
       r.is_integral = first.is_integral && second.is_integral &&
                       third.is_integral;
@@ -775,9 +1029,12 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    }
 
    case nir_op_flrp: {
-      const struct ssa_result_range first = analyze_expression(alu, 0, ht);
-      const struct ssa_result_range second = analyze_expression(alu, 1, ht);
-      const struct ssa_result_range third = analyze_expression(alu, 2, ht);
+      const struct ssa_result_range first =
+         analyze_expression(alu, 0, ht, nir_alu_src_type(alu, 0));
+      const struct ssa_result_range second =
+         analyze_expression(alu, 1, ht, nir_alu_src_type(alu, 1));
+      const struct ssa_result_range third =
+         analyze_expression(alu, 2, ht, nir_alu_src_type(alu, 2));
 
       r.is_integral = first.is_integral && second.is_integral &&
                       third.is_integral;
@@ -801,20 +1058,16 @@ analyze_expression(const nir_alu_instr *instr, unsigned src,
    if (r.range == eq_zero)
       r.is_integral = true;
 
-   _mesa_hash_table_insert(ht, alu, pack_data(r));
+   _mesa_hash_table_insert(ht, pack_key(alu, use_type), pack_data(r));
    return r;
 }
 
 #undef _______
 
 struct ssa_result_range
-nir_analyze_range(const nir_alu_instr *instr, unsigned src)
+nir_analyze_range(struct hash_table *range_ht,
+                  const nir_alu_instr *instr, unsigned src)
 {
-   struct hash_table *ht = _mesa_pointer_hash_table_create(NULL);
-
-   const struct ssa_result_range r = analyze_expression(instr, src, ht);
-
-   _mesa_hash_table_destroy(ht, NULL);
-
-   return r;
+   return analyze_expression(instr, src, range_ht,
+                             nir_alu_src_type(instr, src));
 }
