@@ -57,6 +57,8 @@ static const char *tgsi_file_names[] =
    "SVIEW",
    "BUFFER",
    "MEMORY",
+   "CONSTBUF",
+   "HWATOMIC",
 };
 
 const char *tgsi_semantic_names[TGSI_SEMANTIC_COUNT] =
@@ -106,6 +108,7 @@ const char *tgsi_semantic_names[TGSI_SEMANTIC_COUNT] =
    "SUBGROUP_GT_MASK",
    "SUBGROUP_LE_MASK",
    "SUBGROUP_LT_MASK",
+   "CS_USER_DATA_AMD",
 };
 
 const char *tgsi_texture_names[TGSI_TEXTURE_COUNT] =
@@ -151,11 +154,14 @@ const char *tgsi_property_names[TGSI_PROPERTY_COUNT] =
    "NUM_CLIPDIST_ENABLED",
    "NUM_CULLDIST_ENABLED",
    "FS_EARLY_DEPTH_STENCIL",
+   "FS_POST_DEPTH_COVERAGE",
    "NEXT_SHADER",
    "CS_FIXED_BLOCK_WIDTH",
    "CS_FIXED_BLOCK_HEIGHT",
    "CS_FIXED_BLOCK_DEPTH",
    "MUL_ZERO_WINS",
+   "VS_BLIT_SGPRS_AMD",
+   "CS_USER_DATA_COMPONENTS_AMD",
 };
 
 const char *tgsi_return_type_names[TGSI_RETURN_TYPE_COUNT] =
@@ -182,6 +188,8 @@ const char *tgsi_interpolate_locations[TGSI_INTERPOLATE_LOC_COUNT] =
    "SAMPLE",
 };
 
+const char *tgsi_invariant_name = "INVARIANT";
+
 const char *tgsi_primitive_names[PIPE_PRIM_MAX] =
 {
    "POINTS",
@@ -223,15 +231,16 @@ const char *tgsi_immediate_type_names[6] =
    "INT64",
 };
 
-const char *tgsi_memory_names[3] =
+const char *tgsi_memory_names[4] =
 {
    "COHERENT",
    "RESTRICT",
    "VOLATILE",
+   "STREAM_CACHE_POLICY",
 };
 
 
-static inline void
+UNUSED static inline void
 tgsi_strings_check(void)
 {
    STATIC_ASSERT(ARRAY_SIZE(tgsi_semantic_names) == TGSI_SEMANTIC_COUNT);
