@@ -78,21 +78,21 @@ fd2_blend_state_create(struct pipe_context *pctx,
 
 	so->rb_colorcontrol = A2XX_RB_COLORCONTROL_ROP_CODE(rop);
 
-	so->rb_blendcontrol_rgb =
+	so->rb_blendcontrol =
 		A2XX_RB_BLEND_CONTROL_COLOR_SRCBLEND(fd_blend_factor(rt->rgb_src_factor)) |
 		A2XX_RB_BLEND_CONTROL_COLOR_COMB_FCN(blend_func(rt->rgb_func)) |
 		A2XX_RB_BLEND_CONTROL_COLOR_DESTBLEND(fd_blend_factor(rt->rgb_dst_factor));
 
-	so->rb_blendcontrol_alpha =
-		A2XX_RB_BLEND_CONTROL_ALPHA_SRCBLEND(fd_blend_factor(rt->alpha_src_factor)) |
+	/* hardware doesn't support SRC_ALPHA_SATURATE for alpha, but it is equivalent to ONE */
+	unsigned alpha_src_factor = rt->alpha_src_factor;
+	if (alpha_src_factor == PIPE_BLENDFACTOR_SRC_ALPHA_SATURATE)
+		alpha_src_factor = PIPE_BLENDFACTOR_ONE;
+
+	so->rb_blendcontrol |=
+		A2XX_RB_BLEND_CONTROL_ALPHA_SRCBLEND(fd_blend_factor(alpha_src_factor)) |
 		A2XX_RB_BLEND_CONTROL_ALPHA_COMB_FCN(blend_func(rt->alpha_func)) |
 		A2XX_RB_BLEND_CONTROL_ALPHA_DESTBLEND(fd_blend_factor(rt->alpha_dst_factor));
 
-	so->rb_blendcontrol_no_alpha_rgb =
-		A2XX_RB_BLEND_CONTROL_COLOR_SRCBLEND(fd_blend_factor(util_blend_dst_alpha_to_one(rt->rgb_src_factor))) |
-		A2XX_RB_BLEND_CONTROL_COLOR_COMB_FCN(blend_func(rt->rgb_func)) |
-		A2XX_RB_BLEND_CONTROL_COLOR_DESTBLEND(fd_blend_factor(util_blend_dst_alpha_to_one(rt->rgb_dst_factor)));
-
 	if (rt->colormask & PIPE_MASK_R)
 		so->rb_colormask |= A2XX_RB_COLOR_MASK_WRITE_RED;
 	if (rt->colormask & PIPE_MASK_G)
