@@ -199,7 +199,7 @@ fd2_emit_state_binning(struct fd_context *ctx, const enum fd_dirty_3d_state dirt
 	if (dirty & (FD_DIRTY_PROG | FD_DIRTY_CONST)) {
 		emit_constants(ring,  VS_CONST_BASE * 4,
 				&ctx->constbuf[PIPE_SHADER_VERTEX],
-				(dirty & FD_DIRTY_PROG) ? ctx->prog.vp : NULL);
+				(dirty & FD_DIRTY_PROG) ? ctx->prog.vs : NULL);
 	}
 
 	if (dirty & FD_DIRTY_VIEWPORT) {
@@ -217,15 +217,9 @@ fd2_emit_state_binning(struct fd_context *ctx, const enum fd_dirty_3d_state dirt
 
 	/* not sure why this is needed */
 	if (dirty & (FD_DIRTY_BLEND | FD_DIRTY_FRAMEBUFFER)) {
-		enum pipe_format format =
-			pipe_surface_format(ctx->batch->framebuffer.cbufs[0]);
-		bool has_alpha = util_format_has_alpha(format);
-
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 		OUT_RING(ring, CP_REG(REG_A2XX_RB_BLEND_CONTROL));
-		OUT_RING(ring, blend->rb_blendcontrol_alpha |
-			COND(has_alpha, blend->rb_blendcontrol_rgb) |
-			COND(!has_alpha, blend->rb_blendcontrol_no_alpha_rgb));
+		OUT_RING(ring, blend->rb_blendcontrol);
 
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 		OUT_RING(ring, CP_REG(REG_A2XX_RB_COLOR_MASK));
@@ -242,7 +236,7 @@ fd2_emit_state(struct fd_context *ctx, const enum fd_dirty_3d_state dirty)
 {
 	struct fd2_blend_stateobj *blend = fd2_blend_stateobj(ctx->blend);
 	struct fd2_zsa_stateobj *zsa = fd2_zsa_stateobj(ctx->zsa);
-	struct fd2_shader_stateobj *fp = ctx->prog.fp;
+	struct fd2_shader_stateobj *fs = ctx->prog.fs;
 	struct fd_ringbuffer *ring = ctx->batch->draw;
 
 	/* NOTE: we probably want to eventually refactor this so each state
@@ -262,7 +256,7 @@ fd2_emit_state(struct fd_context *ctx, const enum fd_dirty_3d_state dirty)
 		struct pipe_stencil_ref *sr = &ctx->stencil_ref;
 		uint32_t val = zsa->rb_depthcontrol;
 
-		if (fp->has_kill)
+		if (fs->has_kill)
 			val &= ~A2XX_RB_DEPTHCONTROL_EARLY_Z_ENABLE;
 
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
@@ -301,6 +295,18 @@ fd2_emit_state(struct fd_context *ctx, const enum fd_dirty_3d_state dirty)
 		OUT_RING(ring, fui(1.0));                /* PA_CL_GB_VERT_DISC_ADJ */
 		OUT_RING(ring, fui(1.0));                /* PA_CL_GB_HORZ_CLIP_ADJ */
 		OUT_RING(ring, fui(1.0));                /* PA_CL_GB_HORZ_DISC_ADJ */
+
+		if (rasterizer->base.offset_tri) {
+			/* TODO: why multiply scale by 2 ? without it deqp test fails
+			 * deqp/piglit tests aren't very precise
+			 */
+			OUT_PKT3(ring, CP_SET_CONSTANT, 5);
+			OUT_RING(ring, CP_REG(REG_A2XX_PA_SU_POLY_OFFSET_FRONT_SCALE));
+			OUT_RING(ring, fui(rasterizer->base.offset_scale * 2.0f)); /* FRONT_SCALE */
+			OUT_RING(ring, fui(rasterizer->base.offset_units));        /* FRONT_OFFSET */
+			OUT_RING(ring, fui(rasterizer->base.offset_scale * 2.0f)); /* BACK_SCALE */
+			OUT_RING(ring, fui(rasterizer->base.offset_units));        /* BACK_OFFSET */
+		}
 	}
 
 	/* NOTE: scissor enabled bit is part of rasterizer state: */
@@ -351,10 +357,10 @@ fd2_emit_state(struct fd_context *ctx, const enum fd_dirty_3d_state dirty)
 	if (dirty & (FD_DIRTY_PROG | FD_DIRTY_CONST)) {
 		emit_constants(ring,  VS_CONST_BASE * 4,
 				&ctx->constbuf[PIPE_SHADER_VERTEX],
-				(dirty & FD_DIRTY_PROG) ? ctx->prog.vp : NULL);
+				(dirty & FD_DIRTY_PROG) ? ctx->prog.vs : NULL);
 		emit_constants(ring, PS_CONST_BASE * 4,
 				&ctx->constbuf[PIPE_SHADER_FRAGMENT],
-				(dirty & FD_DIRTY_PROG) ? ctx->prog.fp : NULL);
+				(dirty & FD_DIRTY_PROG) ? ctx->prog.fs : NULL);
 	}
 
 	if (dirty & (FD_DIRTY_BLEND | FD_DIRTY_ZSA)) {
@@ -364,15 +370,9 @@ fd2_emit_state(struct fd_context *ctx, const enum fd_dirty_3d_state dirty)
 	}
 
 	if (dirty & (FD_DIRTY_BLEND | FD_DIRTY_FRAMEBUFFER)) {
-		enum pipe_format format =
-			pipe_surface_format(ctx->batch->framebuffer.cbufs[0]);
-		bool has_alpha = util_format_has_alpha(format);
-
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 		OUT_RING(ring, CP_REG(REG_A2XX_RB_BLEND_CONTROL));
-		OUT_RING(ring, blend->rb_blendcontrol_alpha |
-			COND(has_alpha, blend->rb_blendcontrol_rgb) |
-			COND(!has_alpha, blend->rb_blendcontrol_no_alpha_rgb));
+		OUT_RING(ring, blend->rb_blendcontrol);
 
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 		OUT_RING(ring, CP_REG(REG_A2XX_RB_COLOR_MASK));
