@@ -52,7 +52,7 @@ static uint32_t fmt2swap(enum pipe_format format)
 	case PIPE_FORMAT_B5G5R5X1_UNORM:
 	case PIPE_FORMAT_B4G4R4A4_UNORM:
 	case PIPE_FORMAT_B4G4R4X4_UNORM:
-	/* TODO probably some more.. */
+	case PIPE_FORMAT_B2G3R3_UNORM:
 		return 1;
 	default:
 		return 0;
@@ -62,7 +62,7 @@ static uint32_t fmt2swap(enum pipe_format format)
 static bool
 use_hw_binning(struct fd_batch *batch)
 {
-	struct fd_gmem_stateobj *gmem = &batch->ctx->gmem;
+	const struct fd_gmem_stateobj *gmem = batch->gmem_state;
 
 	/* we hardcoded a limit of 8 "pipes", we can increase this limit
 	 * at the cost of a slightly larger command stream
@@ -89,11 +89,10 @@ emit_gmem2mem_surf(struct fd_batch *batch, uint32_t base,
 {
 	struct fd_ringbuffer *ring = batch->tile_fini;
 	struct fd_resource *rsc = fd_resource(psurf->texture);
-	uint32_t swap = fmt2swap(psurf->format);
-	struct fd_resource_slice *slice =
-		fd_resource_slice(rsc, psurf->u.tex.level);
+	struct fdl_slice *slice = fd_resource_slice(rsc, psurf->u.tex.level);
 	uint32_t offset =
 		fd_resource_offset(rsc, psurf->u.tex.level, psurf->u.tex.first_layer);
+	enum pipe_format format = fd_gmem_restore_format(psurf->format);
 
 	assert((slice->pitch & 31) == 0);
 	assert((offset & 0xfff) == 0);
@@ -103,9 +102,8 @@ emit_gmem2mem_surf(struct fd_batch *batch, uint32_t base,
 
 	OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 	OUT_RING(ring, CP_REG(REG_A2XX_RB_COLOR_INFO));
-	OUT_RING(ring, A2XX_RB_COLOR_INFO_SWAP(swap) |
-			A2XX_RB_COLOR_INFO_BASE(base) |
-			A2XX_RB_COLOR_INFO_FORMAT(fd2_pipe2color(psurf->format)));
+	OUT_RING(ring, A2XX_RB_COLOR_INFO_BASE(base) |
+			A2XX_RB_COLOR_INFO_FORMAT(fd2_pipe2color(format)));
 
 	OUT_PKT3(ring, CP_SET_CONSTANT, 5);
 	OUT_RING(ring, CP_REG(REG_A2XX_RB_COPY_CONTROL));
@@ -113,9 +111,8 @@ emit_gmem2mem_surf(struct fd_batch *batch, uint32_t base,
 	OUT_RELOCW(ring, rsc->bo, offset, 0, 0);     /* RB_COPY_DEST_BASE */
 	OUT_RING(ring, slice->pitch >> 5); /* RB_COPY_DEST_PITCH */
 	OUT_RING(ring,                          /* RB_COPY_DEST_INFO */
-			A2XX_RB_COPY_DEST_INFO_FORMAT(fd2_pipe2color(psurf->format)) |
-			COND(!rsc->tile_mode, A2XX_RB_COPY_DEST_INFO_LINEAR) |
-			A2XX_RB_COPY_DEST_INFO_SWAP(swap) |
+			A2XX_RB_COPY_DEST_INFO_FORMAT(fd2_pipe2color(format)) |
+			COND(!rsc->layout.tile_mode, A2XX_RB_COPY_DEST_INFO_LINEAR) |
 			A2XX_RB_COPY_DEST_INFO_WRITE_RED |
 			A2XX_RB_COPY_DEST_INFO_WRITE_GREEN |
 			A2XX_RB_COPY_DEST_INFO_WRITE_BLUE |
@@ -139,7 +136,7 @@ prepare_tile_fini_ib(struct fd_batch *batch)
 {
 	struct fd_context *ctx = batch->ctx;
 	struct fd2_context *fd2_ctx = fd2_context(ctx);
-	struct fd_gmem_stateobj *gmem = &ctx->gmem;
+	const struct fd_gmem_stateobj *gmem = batch->gmem_state;
 	struct pipe_framebuffer_state *pfb = &batch->framebuffer;
 	struct fd_ringbuffer *ring;
 
@@ -219,7 +216,7 @@ prepare_tile_fini_ib(struct fd_batch *batch)
 }
 
 static void
-fd2_emit_tile_gmem2mem(struct fd_batch *batch, struct fd_tile *tile)
+fd2_emit_tile_gmem2mem(struct fd_batch *batch, const struct fd_tile *tile)
 {
 	fd2_emit_ib(batch->gmem, batch->tile_fini);
 }
@@ -232,20 +229,15 @@ emit_mem2gmem_surf(struct fd_batch *batch, uint32_t base,
 {
 	struct fd_ringbuffer *ring = batch->gmem;
 	struct fd_resource *rsc = fd_resource(psurf->texture);
-	struct fd_resource_slice *slice =
-		fd_resource_slice(rsc, psurf->u.tex.level);
+	struct fdl_slice *slice = fd_resource_slice(rsc, psurf->u.tex.level);
 	uint32_t offset =
 		fd_resource_offset(rsc, psurf->u.tex.level, psurf->u.tex.first_layer);
-	uint32_t swiz;
+	enum pipe_format format = fd_gmem_restore_format(psurf->format);
 
 	OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 	OUT_RING(ring, CP_REG(REG_A2XX_RB_COLOR_INFO));
-	OUT_RING(ring, A2XX_RB_COLOR_INFO_SWAP(fmt2swap(psurf->format)) |
-			A2XX_RB_COLOR_INFO_BASE(base) |
-			A2XX_RB_COLOR_INFO_FORMAT(fd2_pipe2color(psurf->format)));
-
-	swiz = fd2_tex_swiz(psurf->format, PIPE_SWIZZLE_X, PIPE_SWIZZLE_Y,
-			PIPE_SWIZZLE_Z, PIPE_SWIZZLE_W);
+	OUT_RING(ring, A2XX_RB_COLOR_INFO_BASE(base) |
+			A2XX_RB_COLOR_INFO_FORMAT(fd2_pipe2color(format)));
 
 	/* emit fb as a texture: */
 	OUT_PKT3(ring, CP_SET_CONSTANT, 7);
@@ -255,12 +247,15 @@ emit_mem2gmem_surf(struct fd_batch *batch, uint32_t base,
 			A2XX_SQ_TEX_0_CLAMP_Z(SQ_TEX_WRAP) |
 			A2XX_SQ_TEX_0_PITCH(slice->pitch));
 	OUT_RELOC(ring, rsc->bo, offset,
-			fd2_pipe2surface(psurf->format) |
+			A2XX_SQ_TEX_1_FORMAT(fd2_pipe2surface(format).format) |
 			A2XX_SQ_TEX_1_CLAMP_POLICY(SQ_TEX_CLAMP_POLICY_OGL), 0);
 	OUT_RING(ring, A2XX_SQ_TEX_2_WIDTH(psurf->width - 1) |
 			A2XX_SQ_TEX_2_HEIGHT(psurf->height - 1));
 	OUT_RING(ring, A2XX_SQ_TEX_3_MIP_FILTER(SQ_TEX_FILTER_BASEMAP) |
-			swiz |
+			A2XX_SQ_TEX_3_SWIZ_X(0) |
+			A2XX_SQ_TEX_3_SWIZ_Y(1) |
+			A2XX_SQ_TEX_3_SWIZ_Z(2) |
+			A2XX_SQ_TEX_3_SWIZ_W(3) |
 			A2XX_SQ_TEX_3_XY_MAG_FILTER(SQ_TEX_FILTER_POINT) |
 			A2XX_SQ_TEX_3_XY_MIN_FILTER(SQ_TEX_FILTER_POINT));
 	OUT_RING(ring, 0x00000000);
@@ -278,11 +273,11 @@ emit_mem2gmem_surf(struct fd_batch *batch, uint32_t base,
 }
 
 static void
-fd2_emit_tile_mem2gmem(struct fd_batch *batch, struct fd_tile *tile)
+fd2_emit_tile_mem2gmem(struct fd_batch *batch, const struct fd_tile *tile)
 {
 	struct fd_context *ctx = batch->ctx;
 	struct fd2_context *fd2_ctx = fd2_context(ctx);
-	struct fd_gmem_stateobj *gmem = &ctx->gmem;
+	const struct fd_gmem_stateobj *gmem = batch->gmem_state;
 	struct fd_ringbuffer *ring = batch->gmem;
 	struct pipe_framebuffer_state *pfb = &batch->framebuffer;
 	unsigned bin_w = tile->bin_w;
@@ -440,8 +435,7 @@ fd2_emit_sysmem_prep(struct fd_batch *batch)
 		return;
 
 	struct fd_resource *rsc = fd_resource(psurf->texture);
-	struct fd_resource_slice *slice =
-		fd_resource_slice(rsc, psurf->u.tex.level);
+	struct fdl_slice *slice = fd_resource_slice(rsc, psurf->u.tex.level);
 	uint32_t offset =
 		fd_resource_offset(rsc, psurf->u.tex.level, psurf->u.tex.first_layer);
 
@@ -457,7 +451,7 @@ fd2_emit_sysmem_prep(struct fd_batch *batch)
 	OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 	OUT_RING(ring, CP_REG(REG_A2XX_RB_COLOR_INFO));
 	OUT_RELOCW(ring, rsc->bo, offset,
-		COND(!rsc->tile_mode, A2XX_RB_COLOR_INFO_LINEAR) |
+		COND(!rsc->layout.tile_mode, A2XX_RB_COLOR_INFO_LINEAR) |
 		A2XX_RB_COLOR_INFO_SWAP(fmt2swap(psurf->format)) |
 		A2XX_RB_COLOR_INFO_FORMAT(fd2_pipe2color(psurf->format)), 0);
 
@@ -484,7 +478,7 @@ fd2_emit_tile_init(struct fd_batch *batch)
 	struct fd_context *ctx = batch->ctx;
 	struct fd_ringbuffer *ring = batch->gmem;
 	struct pipe_framebuffer_state *pfb = &batch->framebuffer;
-	struct fd_gmem_stateobj *gmem = &ctx->gmem;
+	const struct fd_gmem_stateobj *gmem = batch->gmem_state;
 	enum pipe_format format = pipe_surface_format(pfb->cbufs[0]);
 	uint32_t reg;
 
@@ -589,16 +583,14 @@ fd2_emit_tile_init(struct fd_batch *batch)
 		OUT_RING(ring, 0x0000000C);
 
 		for (int i = 0; i < gmem->num_vsc_pipes; i++) {
-			struct fd_vsc_pipe *pipe = &ctx->vsc_pipe[i];
-
 			/* allocate in 64k increments to avoid reallocs */
 			uint32_t bo_size = align(batch->num_vertices, 0x10000);
-			if (!pipe->bo || fd_bo_size(pipe->bo) < bo_size) {
-				if (pipe->bo)
-					fd_bo_del(pipe->bo);
-				pipe->bo = fd_bo_new(ctx->dev, bo_size,
+			if (!ctx->vsc_pipe_bo[i] || fd_bo_size(ctx->vsc_pipe_bo[i]) < bo_size) {
+				if (ctx->vsc_pipe_bo[i])
+					fd_bo_del(ctx->vsc_pipe_bo[i]);
+				ctx->vsc_pipe_bo[i] = fd_bo_new(ctx->dev, bo_size,
 						DRM_FREEDRENO_GEM_TYPE_KMEM, "vsc_pipe[%u]", i);
-				assert(pipe->bo);
+				assert(ctx->vsc_pipe_bo[i]);
 			}
 
 			/* memory export address (export32):
@@ -607,7 +599,7 @@ fd2_emit_tile_init(struct fd_batch *batch)
 			 * .z: 0x4B00D000 (?)
 			 * .w: 0x4B000000 (?) | max_index (?)
 			*/
-			OUT_RELOCW(ring, pipe->bo, 0, 0x40000000, -2);
+			OUT_RELOCW(ring, ctx->vsc_pipe_bo[i], 0, 0x40000000, -2);
 			OUT_RING(ring, 0x00000000);
 			OUT_RING(ring, 0x4B00D000);
 			OUT_RING(ring, 0x4B000000 | bo_size);
@@ -617,7 +609,7 @@ fd2_emit_tile_init(struct fd_batch *batch)
 		OUT_RING(ring, 0x0000018C);
 
 		for (int i = 0; i < gmem->num_vsc_pipes; i++) {
-			struct fd_vsc_pipe *pipe = &ctx->vsc_pipe[i];
+			const struct fd_vsc_pipe *pipe = &gmem->vsc_pipe[i];
 			float off_x, off_y, mul_x, mul_y;
 
 			/* const to tranform from [-1,1] to bin coordinates for this pipe
@@ -663,7 +655,7 @@ fd2_emit_tile_init(struct fd_batch *batch)
 
 /* before mem2gmem */
 static void
-fd2_emit_tile_prep(struct fd_batch *batch, struct fd_tile *tile)
+fd2_emit_tile_prep(struct fd_batch *batch, const struct fd_tile *tile)
 {
 	struct fd_ringbuffer *ring = batch->gmem;
 	struct pipe_framebuffer_state *pfb = &batch->framebuffer;
@@ -685,7 +677,7 @@ fd2_emit_tile_prep(struct fd_batch *batch, struct fd_tile *tile)
 
 /* before IB to rendering cmds: */
 static void
-fd2_emit_tile_renderprep(struct fd_batch *batch, struct fd_tile *tile)
+fd2_emit_tile_renderprep(struct fd_batch *batch, const struct fd_tile *tile)
 {
 	struct fd_context *ctx = batch->ctx;
 	struct fd2_context *fd2_ctx = fd2_context(ctx);
@@ -729,7 +721,7 @@ fd2_emit_tile_renderprep(struct fd_batch *batch, struct fd_tile *tile)
 	}
 
 	if (use_hw_binning(batch)) {
-		struct fd_vsc_pipe *pipe = &ctx->vsc_pipe[tile->p];
+		struct fd_bo *pipe_bo = ctx->vsc_pipe_bo[tile->p];
 
 		OUT_PKT3(ring, CP_SET_CONSTANT, 2);
 		OUT_RING(ring, CP_REG(REG_A2XX_VGT_CURRENT_BIN_ID_MIN));
@@ -741,7 +733,7 @@ fd2_emit_tile_renderprep(struct fd_batch *batch, struct fd_tile *tile)
 
 		/* TODO only emit this when tile->p changes */
 		OUT_PKT3(ring, CP_SET_DRAW_INIT_FLAGS, 1);
-		OUT_RELOC(ring, pipe->bo, 0, 0, 0);
+		OUT_RELOC(ring, pipe_bo, 0, 0, 0);
 	}
 }
 
