@@ -28,7 +28,7 @@
 #include "util/u_string.h"
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 
 #include "fd3_texture.h"
 #include "fd3_format.h"
@@ -127,13 +127,20 @@ fd3_sampler_state_create(struct pipe_context *pctx,
 	if (cso->compare_mode)
 		so->texsamp0 |= A3XX_TEX_SAMP_0_COMPARE_FUNC(cso->compare_func); /* maps 1:1 */
 
+	so->texsamp1 = A3XX_TEX_SAMP_1_LOD_BIAS(cso->lod_bias);
+
 	if (cso->min_mip_filter != PIPE_TEX_MIPFILTER_NONE) {
-		so->texsamp1 =
-				A3XX_TEX_SAMP_1_LOD_BIAS(cso->lod_bias) |
-				A3XX_TEX_SAMP_1_MIN_LOD(cso->min_lod) |
-				A3XX_TEX_SAMP_1_MAX_LOD(cso->max_lod);
+		so->texsamp1 |=
+			A3XX_TEX_SAMP_1_MIN_LOD(cso->min_lod) |
+			A3XX_TEX_SAMP_1_MAX_LOD(cso->max_lod);
 	} else {
-		so->texsamp1 = 0x00000000;
+		/* If we're not doing mipmap filtering, we still need a slightly > 0
+		 * LOD clamp so the HW can decide between min and mag filtering of
+		 * level 0.
+		 */
+		so->texsamp1 |=
+			A3XX_TEX_SAMP_1_MIN_LOD(MIN2(cso->min_lod, 0.125)) |
+			A3XX_TEX_SAMP_1_MAX_LOD(MIN2(cso->max_lod, 0.125));
 	}
 
 	return so;
@@ -215,7 +222,6 @@ fd3_sampler_view_create(struct pipe_context *pctx, struct pipe_resource *prsc,
 	struct fd3_pipe_sampler_view *so = CALLOC_STRUCT(fd3_pipe_sampler_view);
 	struct fd_resource *rsc = fd_resource(prsc);
 	unsigned lvl;
-	uint32_t sz2 = 0;
 
 	if (!so)
 		return NULL;
@@ -227,7 +233,7 @@ fd3_sampler_view_create(struct pipe_context *pctx, struct pipe_resource *prsc,
 	so->base.context = pctx;
 
 	so->texconst0 =
-	 		A3XX_TEX_CONST_0_TILE_MODE(rsc->tile_mode) |
+			A3XX_TEX_CONST_0_TILE_MODE(rsc->layout.tile_mode) |
 			A3XX_TEX_CONST_0_TYPE(tex_type(prsc->target)) |
 			A3XX_TEX_CONST_0_FMT(fd3_pipe2tex(cso->format)) |
 			fd3_tex_swiz(cso->format, cso->swizzle_r, cso->swizzle_g,
@@ -257,22 +263,22 @@ fd3_sampler_view_create(struct pipe_context *pctx, struct pipe_resource *prsc,
 			A3XX_TEX_CONST_1_HEIGHT(u_minify(prsc->height0, lvl));
 	}
 	/* when emitted, A3XX_TEX_CONST_2_INDX() must be OR'd in: */
+	struct fdl_slice *slice = fd_resource_slice(rsc, lvl);
 	so->texconst2 =
-			A3XX_TEX_CONST_2_PITCH(fd3_pipe2nblocksx(cso->format, rsc->slices[lvl].pitch) * rsc->cpp);
+			A3XX_TEX_CONST_2_PITCH(fd3_pipe2nblocksx(cso->format, slice->pitch) * rsc->layout.cpp);
 	switch (prsc->target) {
 	case PIPE_TEXTURE_1D_ARRAY:
 	case PIPE_TEXTURE_2D_ARRAY:
 		so->texconst3 =
 				A3XX_TEX_CONST_3_DEPTH(prsc->array_size - 1) |
-				A3XX_TEX_CONST_3_LAYERSZ1(rsc->slices[0].size0);
+				A3XX_TEX_CONST_3_LAYERSZ1(slice->size0);
 		break;
 	case PIPE_TEXTURE_3D:
 		so->texconst3 =
 				A3XX_TEX_CONST_3_DEPTH(u_minify(prsc->depth0, lvl)) |
-				A3XX_TEX_CONST_3_LAYERSZ1(rsc->slices[lvl].size0);
-		while (lvl < cso->u.tex.last_level && sz2 != rsc->slices[lvl+1].size0)
-			sz2 = rsc->slices[++lvl].size0;
-		so->texconst3 |= A3XX_TEX_CONST_3_LAYERSZ2(sz2);
+				A3XX_TEX_CONST_3_LAYERSZ1(slice->size0);
+		so->texconst3 |= A3XX_TEX_CONST_3_LAYERSZ2(
+				fd_resource_slice(rsc, prsc->last_level)->size0);
 		break;
 	default:
 		so->texconst3 = 0x00000000;
