@@ -1,5 +1,3 @@
-/* -*- mode: C; c-file-style: "k&r"; tab-width 4; indent-tabs-mode: t; -*- */
-
 /*
  * Copyright (C) 2014 Rob Clark <robclark@freedesktop.org>
  *
@@ -87,19 +85,6 @@ fd4_draw(struct fd_batch *batch, struct fd_ringbuffer *ring,
 	fd_reset_wfi(batch);
 }
 
-static inline enum a4xx_index_size
-fd4_size2indextype(unsigned index_size)
-{
-	switch (index_size) {
-	case 1: return INDEX4_SIZE_8_BIT;
-	case 2: return INDEX4_SIZE_16_BIT;
-	case 4: return INDEX4_SIZE_32_BIT;
-	}
-	DBG("unsupported index size: %d", index_size);
-	assert(0);
-	return INDEX4_SIZE_32_BIT;
-}
-
 static inline void
 fd4_draw_emit(struct fd_batch *batch, struct fd_ringbuffer *ring,
 		enum pc_di_primtype primtype,
