@@ -25,15 +25,16 @@
  */
 
 #include "pipe/p_screen.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 
 #include "fd5_screen.h"
 #include "fd5_blitter.h"
 #include "fd5_context.h"
 #include "fd5_format.h"
+#include "fd5_emit.h"
 #include "fd5_resource.h"
 
-#include "ir3_compiler.h"
+#include "ir3/ir3_compiler.h"
 
 static bool
 valid_sample_count(unsigned sample_count)
@@ -49,7 +50,7 @@ valid_sample_count(unsigned sample_count)
 	}
 }
 
-static boolean
+static bool
 fd5_screen_is_format_supported(struct pipe_screen *pscreen,
 		enum pipe_format format,
 		enum pipe_texture_target target,
@@ -63,7 +64,7 @@ fd5_screen_is_format_supported(struct pipe_screen *pscreen,
 			!valid_sample_count(sample_count)) {
 		DBG("not supported: format=%s, target=%d, sample_count=%d, usage=%x",
 				util_format_name(format), target, sample_count, usage);
-		return FALSE;
+		return false;
 	}
 
 	if (MAX2(1, sample_count) != MAX2(1, storage_sample_count))
@@ -75,9 +76,9 @@ fd5_screen_is_format_supported(struct pipe_screen *pscreen,
 	}
 
 	if ((usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE)) &&
+			(fd5_pipe2tex(format) != (enum a5xx_tex_fmt)~0) &&
 			(target == PIPE_BUFFER ||
-			 util_format_get_blocksize(format) != 12) &&
-			(fd5_pipe2tex(format) != (enum a5xx_tex_fmt)~0)) {
+			 util_format_get_blocksize(format) != 12)) {
 		retval |= usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE);
 	}
 
@@ -120,9 +121,6 @@ fd5_screen_is_format_supported(struct pipe_screen *pscreen,
 	return retval == usage;
 }
 
-extern const struct fd_perfcntr_group a5xx_perfcntr_groups[];
-extern const unsigned a5xx_num_perfcntr_groups;
-
 void
 fd5_screen_init(struct pipe_screen *pscreen)
 {
@@ -136,8 +134,5 @@ fd5_screen_init(struct pipe_screen *pscreen)
 	if (fd_mesa_debug & FD_DBG_TTILE)
 		screen->tile_mode = fd5_tile_mode;
 
-	if (fd_mesa_debug & FD_DBG_PERFC) {
-		screen->perfcntr_groups = a5xx_perfcntr_groups;
-		screen->num_perfcntr_groups = a5xx_num_perfcntr_groups;
-	}
+	fd5_emit_init_screen(pscreen);
 }
