@@ -29,6 +29,7 @@
 
 #include "drm/freedreno_drmif.h"
 #include "drm/freedreno_ringbuffer.h"
+#include "perfcntrs/freedreno_perfcntr.h"
 
 #include "pipe/p_screen.h"
 #include "util/u_memory.h"
@@ -37,7 +38,7 @@
 #include "renderonly/renderonly.h"
 
 #include "freedreno_batch_cache.h"
-#include "freedreno_perfcntr.h"
+#include "freedreno_gmem.h"
 #include "freedreno_util.h"
 
 struct fd_bo;
@@ -113,6 +114,7 @@ struct fd_screen {
 	int64_t cpu_gpu_time_delta;
 
 	struct fd_batch_cache batch_cache;
+	struct fd_gmem_cache gmem_cache;
 
 	bool reorder;
 
