@@ -25,7 +25,7 @@
 #include "r300_texture.h"
 #include "r300_reg.h"
 
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_half.h"
 #include "util/u_pack_color.h"
 #include "util/u_surface.h"
