@@ -54,11 +54,11 @@ static struct r600_bytecode_cf *r600_bytecode_cf(void)
 
 	if (!cf)
 		return NULL;
-	LIST_INITHEAD(&cf->list);
-	LIST_INITHEAD(&cf->alu);
-	LIST_INITHEAD(&cf->vtx);
-	LIST_INITHEAD(&cf->tex);
-	LIST_INITHEAD(&cf->gds);
+	list_inithead(&cf->list);
+	list_inithead(&cf->alu);
+	list_inithead(&cf->vtx);
+	list_inithead(&cf->tex);
+	list_inithead(&cf->gds);
 	return cf;
 }
 
@@ -68,7 +68,7 @@ static struct r600_bytecode_alu *r600_bytecode_alu(void)
 
 	if (!alu)
 		return NULL;
-	LIST_INITHEAD(&alu->list);
+	list_inithead(&alu->list);
 	return alu;
 }
 
@@ -78,7 +78,7 @@ static struct r600_bytecode_vtx *r600_bytecode_vtx(void)
 
 	if (!vtx)
 		return NULL;
-	LIST_INITHEAD(&vtx->list);
+	list_inithead(&vtx->list);
 	return vtx;
 }
 
@@ -88,7 +88,7 @@ static struct r600_bytecode_tex *r600_bytecode_tex(void)
 
 	if (!tex)
 		return NULL;
-	LIST_INITHEAD(&tex->list);
+	list_inithead(&tex->list);
 	return tex;
 }
 
@@ -98,7 +98,7 @@ static struct r600_bytecode_gds *r600_bytecode_gds(void)
 
 	if (gds == NULL)
 		return NULL;
-	LIST_INITHEAD(&gds->list);
+	list_inithead(&gds->list);
 	return gds;
 }
 
@@ -154,7 +154,7 @@ void r600_bytecode_init(struct r600_bytecode *bc,
 		bc->r6xx_nop_after_rel_dst = 0;
 	}
 
-	LIST_INITHEAD(&bc->cf);
+	list_inithead(&bc->cf);
 	bc->chip_class = chip_class;
 	bc->family = family;
 	bc->has_compressed_msaa_texturing = has_compressed_msaa_texturing;
@@ -167,7 +167,7 @@ int r600_bytecode_add_cf(struct r600_bytecode *bc)
 
 	if (!cf)
 		return -ENOMEM;
-	LIST_ADDTAIL(&cf->list, &bc->cf);
+	list_addtail(&cf->list, &bc->cf);
 	if (bc->cf_last) {
 		cf->id = bc->cf_last->id + 2;
 		if (bc->cf_last->eg_alu_extended) {
@@ -928,9 +928,9 @@ static int merge_inst_groups(struct r600_bytecode *bc, struct r600_bytecode_alu
 	for (i = 0; i < max_slots; ++i) {
 		slots[i] = result[i];
 		if (result[i]) {
-			LIST_DEL(&result[i]->list);
+			list_del(&result[i]->list);
 			result[i]->last = 0;
-			LIST_ADDTAIL(&result[i]->list, &bc->cf_last->alu);
+			list_addtail(&result[i]->list, &bc->cf_last->alu);
 		}
 	}
 
@@ -1266,7 +1266,7 @@ int r600_bytecode_add_alu_type(struct r600_bytecode *bc,
 	if (nalu->dst.sel >= bc->ngpr) {
 		bc->ngpr = nalu->dst.sel + 1;
 	}
-	LIST_ADDTAIL(&nalu->list, &bc->cf_last->alu);
+	list_addtail(&nalu->list, &bc->cf_last->alu);
 	/* each alu use 2 dwords */
 	bc->cf_last->ndw += 2;
 	bc->ndw += 2;
@@ -1407,7 +1407,7 @@ static int r600_bytecode_add_vtx_internal(struct r600_bytecode *bc, const struct
 			return -EINVAL;
 		}
 	}
-	LIST_ADDTAIL(&nvtx->list, &bc->cf_last->vtx);
+	list_addtail(&nvtx->list, &bc->cf_last->vtx);
 	/* each fetch use 4 dwords */
 	bc->cf_last->ndw += 4;
 	bc->ndw += 4;
@@ -1477,7 +1477,7 @@ int r600_bytecode_add_tex(struct r600_bytecode *bc, const struct r600_bytecode_t
 	if (ntex->dst_gpr >= bc->ngpr) {
 		bc->ngpr = ntex->dst_gpr + 1;
 	}
-	LIST_ADDTAIL(&ntex->list, &bc->cf_last->tex);
+	list_addtail(&ntex->list, &bc->cf_last->tex);
 	/* each texture fetch use 4 dwords */
 	bc->cf_last->ndw += 4;
 	bc->ndw += 4;
@@ -1511,7 +1511,7 @@ int r600_bytecode_add_gds(struct r600_bytecode *bc, const struct r600_bytecode_g
 		bc->cf_last->op = CF_OP_GDS;
 	}
 
-	LIST_ADDTAIL(&ngds->list, &bc->cf_last->gds);
+	list_addtail(&ngds->list, &bc->cf_last->gds);
 	bc->cf_last->ndw += 4; /* each GDS uses 4 dwords */
 	if ((bc->cf_last->ndw / 4) >= r600_bytecode_num_tex_and_vtx_instructions(bc))
 		bc->force_add_cf = 1;
@@ -1867,30 +1867,30 @@ void r600_bytecode_clear(struct r600_bytecode *bc)
 			free(alu);
 		}
 
-		LIST_INITHEAD(&cf->alu);
+		list_inithead(&cf->alu);
 
 		LIST_FOR_EACH_ENTRY_SAFE(tex, next_tex, &cf->tex, list) {
 			free(tex);
 		}
 
-		LIST_INITHEAD(&cf->tex);
+		list_inithead(&cf->tex);
 
 		LIST_FOR_EACH_ENTRY_SAFE(vtx, next_vtx, &cf->vtx, list) {
 			free(vtx);
 		}
 
-		LIST_INITHEAD(&cf->vtx);
+		list_inithead(&cf->vtx);
 
 		LIST_FOR_EACH_ENTRY_SAFE(gds, next_gds, &cf->gds, list) {
 			free(gds);
 		}
 
-		LIST_INITHEAD(&cf->gds);
+		list_inithead(&cf->gds);
 
 		free(cf);
 	}
 
-	LIST_INITHEAD(&cf->list);
+	list_inithead(&cf->list);
 }
 
 static int print_swizzle(unsigned swz)
@@ -2476,6 +2476,11 @@ void r600_vertex_data_type(enum pipe_format pformat,
 		return;
 	}
 
+	if (pformat == PIPE_FORMAT_A1B5G5R5_UNORM) {
+		*format = FMT_5_5_5_1;
+		return;
+	}
+
 	desc = util_format_description(pformat);
 	if (desc->layout != UTIL_FORMAT_LAYOUT_PLAIN) {
 		goto out_unknown;
@@ -2533,6 +2538,16 @@ void r600_vertex_data_type(enum pipe_format pformat,
 		/* Signed ints */
 	case UTIL_FORMAT_TYPE_SIGNED:
 		switch (desc->channel[i].size) {
+		case 4:
+			switch (desc->nr_channels) {
+			case 2:
+				*format = FMT_4_4;
+				break;
+			case 4:
+				*format = FMT_4_4_4_4;
+				break;
+			}
+			break;
 		case 8:
 			switch (desc->nr_channels) {
 			case 1:
@@ -2757,7 +2772,9 @@ void *r600_create_vertex_fetch_shader(struct pipe_context *ctx,
 		return NULL;
 	}
 
-	bytecode = r600_buffer_map_sync_with_rings(&rctx->b, shader->buffer, PIPE_TRANSFER_WRITE | PIPE_TRANSFER_UNSYNCHRONIZED);
+	bytecode = r600_buffer_map_sync_with_rings
+		(&rctx->b, shader->buffer,
+		PIPE_TRANSFER_WRITE | PIPE_TRANSFER_UNSYNCHRONIZED | RADEON_TRANSFER_TEMPORARY);
 	bytecode += shader->offset / 4;
 
 	if (R600_BIG_ENDIAN) {
