@@ -30,7 +30,7 @@
 #include "util/u_inlines.h"
 #include "os/os_thread.h"
 #include "util/u_bitmask.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
 
