@@ -44,10 +44,13 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
    struct pipe_transfer *ib_transfer = NULL;
    struct pipe_transfer *cb_transfer[SVGA_MAX_CONST_BUFS] = { 0 };
    struct draw_context *draw = svga->swtnl.draw;
-   unsigned i, old_num_vertex_buffers;
+   ASSERTED unsigned old_num_vertex_buffers;
+   unsigned i;
    const void *map;
    enum pipe_error ret;
 
+   SVGA_STATS_TIME_PUSH(svga_sws(svga), SVGA_STATS_TIME_SWTNLDRAWVBO);
+
    assert(!svga->dirty);
    assert(svga->state.sw.need_swtnl);
    assert(draw);
@@ -67,10 +70,11 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
     * Map vertex buffers
     */
    for (i = 0; i < svga->curr.num_vertex_buffers; i++) {
-      if (svga->curr.vb[i].buffer) {
+      if (svga->curr.vb[i].buffer.resource) {
          map = pipe_buffer_map(&svga->pipe,
-                               svga->curr.vb[i].buffer,
-                               PIPE_TRANSFER_READ,
+                               svga->curr.vb[i].buffer.resource,
+                               PIPE_TRANSFER_READ |
+                               PIPE_TRANSFER_UNSYNCHRONIZED,
                                &vb_transfer[i]);
 
          draw_set_mapped_vertex_buffer(draw, i, map, ~0);
@@ -80,24 +84,29 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
 
    /* Map index buffer, if present */
    map = NULL;
-   if (info->indexed && svga->curr.ib.buffer) {
-      map = pipe_buffer_map(&svga->pipe, svga->curr.ib.buffer,
-                            PIPE_TRANSFER_READ,
-                            &ib_transfer);
+   if (info->index_size) {
+      if (info->has_user_indices) {
+         map = (ubyte *) info->index.user;
+      } else {
+         map = pipe_buffer_map(&svga->pipe, info->index.resource,
+                               PIPE_TRANSFER_READ |
+                               PIPE_TRANSFER_UNSYNCHRONIZED, &ib_transfer);
+      }
       draw_set_indexes(draw,
-                       (const ubyte *) map + svga->curr.ib.offset,
-                       svga->curr.ib.index_size, ~0);
+                       (const ubyte *) map,
+                       info->index_size, ~0);
    }
 
    /* Map constant buffers */
-   for (i = 0; i < Elements(svga->curr.constbufs[PIPE_SHADER_VERTEX]); ++i) {
+   for (i = 0; i < ARRAY_SIZE(svga->curr.constbufs[PIPE_SHADER_VERTEX]); ++i) {
       if (svga->curr.constbufs[PIPE_SHADER_VERTEX][i].buffer == NULL) {
          continue;
       }
 
       map = pipe_buffer_map(&svga->pipe,
                             svga->curr.constbufs[PIPE_SHADER_VERTEX][i].buffer,
-                            PIPE_TRANSFER_READ,
+                            PIPE_TRANSFER_READ |
+                            PIPE_TRANSFER_UNSYNCHRONIZED,
                             &cb_transfer[i]);
       assert(map);
       draw_set_mapped_constant_buffer(
@@ -117,7 +126,7 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
     * unmap vertex/index buffers
     */
    for (i = 0; i < svga->curr.num_vertex_buffers; i++) {
-      if (svga->curr.vb[i].buffer) {
+      if (svga->curr.vb[i].buffer.resource) {
          pipe_buffer_unmap(&svga->pipe, vb_transfer[i]);
          draw_set_mapped_vertex_buffer(draw, i, NULL, 0);
       }
@@ -128,7 +137,7 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
       draw_set_indexes(draw, NULL, 0, 0);
    }
 
-   for (i = 0; i < Elements(svga->curr.constbufs[PIPE_SHADER_VERTEX]); ++i) {
+   for (i = 0; i < ARRAY_SIZE(svga->curr.constbufs[PIPE_SHADER_VERTEX]); ++i) {
       if (svga->curr.constbufs[PIPE_SHADER_VERTEX][i].buffer) {
          pipe_buffer_unmap(&svga->pipe, cb_transfer[i]);
       }
@@ -138,18 +147,18 @@ svga_swtnl_draw_vbo(struct svga_context *svga,
    svga->state.sw.in_swtnl_draw = FALSE;
    svga->dirty |= SVGA_NEW_NEED_PIPELINE | SVGA_NEW_NEED_SWVFETCH;
 
+   SVGA_STATS_TIME_POP(svga_sws(svga));
    return ret;
 }
 
 
-
-
-boolean svga_init_swtnl( struct svga_context *svga )
+boolean
+svga_init_swtnl(struct svga_context *svga)
 {
    struct svga_screen *screen = svga_screen(svga->pipe.screen);
 
    svga->swtnl.backend = svga_vbuf_render_create(svga);
-   if(!svga->swtnl.backend)
+   if (!svga->swtnl.backend)
       goto fail;
 
    /*
@@ -160,8 +169,8 @@ boolean svga_init_swtnl( struct svga_context *svga )
       goto fail;
 
 
-   draw_set_rasterize_stage(svga->swtnl.draw, 
-                            draw_vbuf_stage( svga->swtnl.draw, svga->swtnl.backend ));
+   draw_set_rasterize_stage(svga->swtnl.draw,
+                 draw_vbuf_stage(svga->swtnl.draw, svga->swtnl.backend));
 
    draw_set_render(svga->swtnl.draw, svga->swtnl.backend);
 
@@ -197,16 +206,17 @@ fail:
       util_blitter_destroy(svga->blitter);
 
    if (svga->swtnl.backend)
-      svga->swtnl.backend->destroy( svga->swtnl.backend );
+      svga->swtnl.backend->destroy(svga->swtnl.backend);
 
    if (svga->swtnl.draw)
-      draw_destroy( svga->swtnl.draw );
+      draw_destroy(svga->swtnl.draw);
 
    return FALSE;
 }
 
 
-void svga_destroy_swtnl( struct svga_context *svga )
+void
+svga_destroy_swtnl(struct svga_context *svga)
 {
-   draw_destroy( svga->swtnl.draw );
+   draw_destroy(svga->swtnl.draw);
 }
