@@ -28,10 +28,11 @@
 
 #include <sys/stat.h>
 
-#include <drm_fourcc.h>
-#include <tegra_drm.h>
+#include "drm-uapi/drm_fourcc.h"
+#include "drm-uapi/tegra_drm.h"
 #include <xf86drm.h>
 
+#include "loader/loader.h"
 #include "pipe/p_state.h"
 #include "util/u_debug.h"
 #include "util/u_inlines.h"
@@ -127,7 +128,7 @@ tegra_screen_get_timestamp(struct pipe_screen *pscreen)
    return screen->gpu->get_timestamp(screen->gpu);
 }
 
-static boolean
+static bool
 tegra_screen_is_format_supported(struct pipe_screen *pscreen,
                                  enum pipe_format format,
                                  enum pipe_texture_target target,
@@ -142,7 +143,7 @@ tegra_screen_is_format_supported(struct pipe_screen *pscreen,
                                            usage);
 }
 
-static boolean
+static bool
 tegra_screen_is_video_format_supported(struct pipe_screen *pscreen,
                                        enum pipe_format format,
                                        enum pipe_video_profile profile,
@@ -154,7 +155,7 @@ tegra_screen_is_video_format_supported(struct pipe_screen *pscreen,
                                                  entrypoint);
 }
 
-static boolean
+static bool
 tegra_screen_can_create_resource(struct pipe_screen *pscreen,
                                  const struct pipe_resource *template)
 {
@@ -163,70 +164,11 @@ tegra_screen_can_create_resource(struct pipe_screen *pscreen,
    return screen->gpu->can_create_resource(screen->gpu, template);
 }
 
-static int tegra_open_render_node(void)
-{
-   drmDevicePtr *devices, device;
-   int err, render = -ENOENT, fd;
-   unsigned int num, i;
-
-   err = drmGetDevices2(0, NULL, 0);
-   if (err < 0)
-      return err;
-
-   num = err;
-
-   devices = calloc(num, sizeof(*devices));
-   if (!devices)
-      return -ENOMEM;
-
-   err = drmGetDevices2(0, devices, num);
-   if (err < 0) {
-      render = err;
-      goto free;
-   }
-
-   for (i = 0; i < num; i++) {
-      device = devices[i];
-
-      if ((device->available_nodes & (1 << DRM_NODE_RENDER)) &&
-          (device->bustype == DRM_BUS_PLATFORM)) {
-         drmVersionPtr version;
-
-         fd = open(device->nodes[DRM_NODE_RENDER], O_RDWR | O_CLOEXEC);
-         if (fd < 0)
-            continue;
-
-         version = drmGetVersion(fd);
-         if (!version) {
-            close(fd);
-            continue;
-         }
-
-         if (strcmp(version->name, "nouveau") != 0) {
-            drmFreeVersion(version);
-            close(fd);
-            continue;
-         }
-
-         drmFreeVersion(version);
-         render = fd;
-         break;
-      }
-   }
-
-   drmFreeDevices(devices, num);
-
-free:
-   free(devices);
-   return render;
-}
-
 static int tegra_screen_import_resource(struct tegra_screen *screen,
                                         struct tegra_resource *resource)
 {
-   unsigned usage = PIPE_HANDLE_USAGE_READ;
    struct winsys_handle handle;
-   boolean status;
+   bool status;
    int fd, err;
 
    memset(&handle, 0, sizeof(handle));
@@ -234,7 +176,7 @@ static int tegra_screen_import_resource(struct tegra_screen *screen,
    handle.type = WINSYS_HANDLE_TYPE_FD;
 
    status = screen->gpu->resource_get_handle(screen->gpu, NULL, resource->gpu,
-                                             &handle, usage);
+                                             &handle, 0);
    if (!status)
       return -EINVAL;
 
@@ -372,7 +314,7 @@ tegra_screen_resource_from_user_memory(struct pipe_screen *pscreen,
    return resource;
 }
 
-static boolean
+static bool
 tegra_screen_resource_get_handle(struct pipe_screen *pscreen,
                                  struct pipe_context *pcontext,
                                  struct pipe_resource *presource,
@@ -382,7 +324,7 @@ tegra_screen_resource_get_handle(struct pipe_screen *pscreen,
    struct tegra_resource *resource = to_tegra_resource(presource);
    struct tegra_context *context = to_tegra_context(pcontext);
    struct tegra_screen *screen = to_tegra_screen(pscreen);
-   boolean ret = TRUE;
+   bool ret = true;
 
    /*
     * Assume that KMS handles for scanout resources will only ever be used
@@ -437,7 +379,7 @@ tegra_screen_fence_reference(struct pipe_screen *pscreen,
    screen->gpu->fence_reference(screen->gpu, ptr, fence);
 }
 
-static boolean
+static bool
 tegra_screen_fence_finish(struct pipe_screen *pscreen,
                           struct pipe_context *pcontext,
                           struct pipe_fence_handle *fence,
@@ -594,7 +536,7 @@ tegra_screen_create(int fd)
 
    screen->fd = fd;
 
-   screen->gpu_fd = tegra_open_render_node();
+   screen->gpu_fd = loader_open_render_node("nouveau");
    if (screen->gpu_fd < 0) {
       if (errno != ENOENT)
          fprintf(stderr, "failed to open GPU device: %s\n", strerror(errno));
