@@ -34,18 +34,6 @@ struct partial_update_state {
         uint8_t channels;
 };
 
-static uint32_t
-int_hash(const void *key)
-{
-        return _mesa_hash_data(key, sizeof(int));
-}
-
-static bool
-int_compare(const void *key1, const void *key2)
-{
-        return *(const int *)key1 == *(const int *)key2;
-}
-
 static int
 qir_reg_to_var(struct qreg reg)
 {
@@ -173,8 +161,6 @@ qir_setup_def(struct vc4_compile *c, struct qblock *block, int ip,
 static void
 sf_state_clear(struct hash_table *partial_update_ht)
 {
-        struct hash_entry *entry;
-
         hash_table_foreach(partial_update_ht, entry) {
                 struct partial_update_state *state = entry->data;
 
@@ -196,7 +182,7 @@ static void
 qir_setup_def_use(struct vc4_compile *c)
 {
         struct hash_table *partial_update_ht =
-                _mesa_hash_table_create(c, int_hash, int_compare);
+                _mesa_hash_table_create(c, _mesa_hash_int, _mesa_key_int_equal);
         int ip = 0;
 
         qir_for_each_block(block, c) {
