@@ -20,14 +20,14 @@
 // OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#include "core/compiler.hpp"
 #include "core/program.hpp"
-#include "llvm/invocation.hpp"
-#include "tgsi/invocation.hpp"
 
 using namespace clover;
 
 program::program(clover::context &ctx, const std::string &source) :
-   has_source(true), context(ctx), _source(source), _kernel_ref_counter(0) {
+   has_source(true), context(ctx), _devices(ctx.devices()), _source(source),
+   _kernel_ref_counter(0) {
 }
 
 program::program(clover::context &ctx,
@@ -51,10 +51,8 @@ program::compile(const ref_vector<device> &devs, const std::string &opts,
          std::string log;
 
          try {
-            const module m = (dev.ir_format() == PIPE_SHADER_IR_TGSI ?
-                              tgsi::compile_program(_source, log) :
-                              llvm::compile_program(_source, headers,
-                                                    dev.ir_target(), opts, log));
+            const module m =
+               compiler::compile_program(_source, headers, dev, opts, log);
             _builds[&dev] = { m, opts, log };
          } catch (...) {
             _builds[&dev] = { module(), opts, log };
@@ -76,10 +74,7 @@ program::link(const ref_vector<device> &devs, const std::string &opts,
       std::string log = _builds[&dev].log;
 
       try {
-         const module m = (dev.ir_format() == PIPE_SHADER_IR_TGSI ?
-                           tgsi::link_program(ms) :
-                           llvm::link_program(ms, dev.ir_format(),
-                                              dev.ir_target(), opts, log));
+         const module m = compiler::link_program(ms, dev, opts, log);
          _builds[&dev] = { m, opts, log };
       } catch (...) {
          _builds[&dev] = { module(), opts, log };
@@ -108,6 +103,18 @@ program::build::status() const {
       return CL_BUILD_NONE;
 }
 
+cl_program_binary_type
+program::build::binary_type() const {
+   if (any_of(type_equals(module::section::text_intermediate), binary.secs))
+      return CL_PROGRAM_BINARY_TYPE_COMPILED_OBJECT;
+   else if (any_of(type_equals(module::section::text_library), binary.secs))
+      return CL_PROGRAM_BINARY_TYPE_LIBRARY;
+   else if (any_of(type_equals(module::section::text_executable), binary.secs))
+      return CL_PROGRAM_BINARY_TYPE_EXECUTABLE;
+   else
+      return CL_PROGRAM_BINARY_TYPE_NONE;
+}
+
 const struct program::build &
 program::build(const device &dev) const {
    static const struct build null;
