@@ -668,7 +668,7 @@ static struct pipe_video_buffer *vid_dec_h265_Flush(vid_dec_PrivateType *priv,
       *timestamp = result->timestamp;
 
    --priv->codec_data.h265.dpb_num;
-   LIST_DEL(&result->list);
+   list_del(&result->list);
    FREE(result);
 
    return buf;
@@ -736,7 +736,7 @@ static void vid_dec_h265_EndFrame(vid_dec_PrivateType *priv)
    entry->timestamp = priv->timestamp;
    entry->poc = get_poc(priv);
 
-   LIST_ADDTAIL(&entry->list, &priv->codec_data.h265.dpb_list);
+   list_addtail(&entry->list, &priv->codec_data.h265.dpb_list);
    ++priv->codec_data.h265.dpb_num;
    priv->target = NULL;
 
@@ -1002,7 +1002,7 @@ void vid_dec_h265_Init(vid_dec_PrivateType *priv)
 {
    priv->picture.base.profile = PIPE_VIDEO_PROFILE_HEVC_MAIN;
 
-   LIST_INITHEAD(&priv->codec_data.h265.dpb_list);
+   list_inithead(&priv->codec_data.h265.dpb_list);
    priv->codec_data.h265.ref_pic_set_list = (struct ref_pic_set *)
       CALLOC(MAX_NUM_REF_PICS, sizeof(struct ref_pic_set));
 
