@@ -432,7 +432,7 @@ static OMX_ERRORTYPE h264d_prc_allocate_resources(void *ap_obj, OMX_U32 a_pid)
       return OMX_ErrorInsufficientResources;
    }
 
-   LIST_INITHEAD(&priv->codec_data.h264.dpb_list);
+   list_inithead(&priv->codec_data.h264.dpb_list);
 
    priv->video_buffer_map = util_hash_table_create(handle_hash, handle_compare);
 
