@@ -268,16 +268,16 @@ static void enc_ClearBframes(vid_enc_PrivateType * priv, struct input_buf_privat
 {
    struct encode_task *task;
 
-   if (LIST_IS_EMPTY(&priv->b_frames))
+   if (list_is_empty(&priv->b_frames))
       return;
 
    task = LIST_ENTRY(struct encode_task, priv->b_frames.prev, list);
-   LIST_DEL(&task->list);
+   list_del(&task->list);
 
    /* promote last from to P frame */
    priv->ref_idx_l0 = priv->ref_idx_l1;
    enc_HandleTask(priv, task, PIPE_H264_ENC_PICTURE_TYPE_P);
-   LIST_ADDTAIL(&task->list, &inp->tasks);
+   list_addtail(&task->list, &inp->tasks);
    priv->ref_idx_l1 = priv->frame_num++;
 
    /* handle B frames */
@@ -354,20 +354,20 @@ static OMX_ERRORTYPE encode_frame(vid_enc_PrivateType * priv, OMX_BUFFERHEADERTY
 
    if (picture_type == PIPE_H264_ENC_PICTURE_TYPE_B) {
       /* put frame at the tail of the queue */
-      LIST_ADDTAIL(&task->list, &priv->b_frames);
+      list_addtail(&task->list, &priv->b_frames);
    } else {
       /* handle I or P frame */
       priv->ref_idx_l0 = priv->ref_idx_l1;
       enc_HandleTask(priv, task, picture_type);
-      LIST_ADDTAIL(&task->list, &priv->stacked_tasks);
+      list_addtail(&task->list, &priv->stacked_tasks);
       LIST_FOR_EACH_ENTRY(task, &priv->stacked_tasks, list) {
          ++stacked_num;
       }
       if (stacked_num == priv->stacked_frames_num) {
          struct encode_task *t;
          t = LIST_ENTRY(struct encode_task, priv->stacked_tasks.next, list);
-         LIST_DEL(&t->list);
-         LIST_ADDTAIL(&t->list, &inp->tasks);
+         list_del(&t->list);
+         list_addtail(&t->list, &inp->tasks);
       }
       priv->ref_idx_l1 = priv->frame_num++;
 
@@ -382,7 +382,7 @@ static OMX_ERRORTYPE encode_frame(vid_enc_PrivateType * priv, OMX_BUFFERHEADERTY
       enc_MoveTasks(&priv->b_frames, &inp->tasks);
    }
 
-   if (LIST_IS_EMPTY(&inp->tasks)) {
+   if (list_is_empty(&inp->tasks)) {
       return h264e_buffer_emptied(priv, in_buf);
    } else {
       return h264e_manage_buffers(priv);
@@ -426,10 +426,10 @@ static OMX_ERRORTYPE h264e_prc_create_encoder(void *ap_obj)
    if (!priv->t_pipe)
       return OMX_ErrorInsufficientResources;
 
-   LIST_INITHEAD(&priv->free_tasks);
-   LIST_INITHEAD(&priv->used_tasks);
-   LIST_INITHEAD(&priv->b_frames);
-   LIST_INITHEAD(&priv->stacked_tasks);
+   list_inithead(&priv->free_tasks);
+   list_inithead(&priv->used_tasks);
+   list_inithead(&priv->b_frames);
+   list_inithead(&priv->stacked_tasks);
 
    return OMX_ErrorNone;
 }
