@@ -29,7 +29,7 @@
 
 #include "util/u_memory.h"
 #include "util/u_debug.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_sampler.h"
 
 #include "vdpau_private.h"
