@@ -31,7 +31,7 @@
 #include "util/u_debug.h"
 #include "util/u_memory.h"
 #include "util/u_sampler.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_surface.h"
 
 #include "vl/vl_csc.h"
