@@ -38,28 +38,23 @@
 void *
 vmw_svga_winsys_surface_map(struct svga_winsys_context *swc,
                             struct svga_winsys_surface *srf,
-                            unsigned flags, boolean *retry)
+                            unsigned flags, boolean *retry,
+                            boolean *rebind)
 {
    struct vmw_svga_winsys_surface *vsrf = vmw_svga_winsys_surface(srf);
    void *data = NULL;
    struct pb_buffer *pb_buf;
    uint32_t pb_flags;
    struct vmw_winsys_screen *vws = vsrf->screen;
-   
+
    *retry = FALSE;
+   *rebind = FALSE;
    assert((flags & (PIPE_TRANSFER_READ | PIPE_TRANSFER_WRITE)) != 0);
-   pipe_mutex_lock(vsrf->mutex);
+   mtx_lock(&vsrf->mutex);
 
    if (vsrf->mapcount) {
-      /*
-       * Only allow multiple readers to map.
-       */
-      if ((flags & PIPE_TRANSFER_WRITE) ||
-          (vsrf->map_mode & PIPE_TRANSFER_WRITE))
-         goto out_unlock;
-      
-      data = vsrf->data;
-      goto out_mapped;
+      /* Other mappers will get confused if we discard. */
+      flags &= ~PIPE_TRANSFER_DISCARD_WHOLE_RESOURCE;
    }
 
    vsrf->rebind = FALSE;
@@ -89,7 +84,8 @@ vmw_svga_winsys_surface_map(struct svga_winsys_context *swc,
       goto out_unlock;
    }
 
-   pb_flags = flags & (PIPE_TRANSFER_READ_WRITE | PIPE_TRANSFER_UNSYNCHRONIZED);
+   pb_flags = flags & (PIPE_TRANSFER_READ_WRITE | PIPE_TRANSFER_UNSYNCHRONIZED |
+      PIPE_TRANSFER_PERSISTENT);
 
    if (flags & PIPE_TRANSFER_DISCARD_WHOLE_RESOURCE) {
       struct pb_manager *provider;
@@ -127,6 +123,12 @@ vmw_svga_winsys_surface_map(struct svga_winsys_context *swc,
             if (vsrf->buf)
                vmw_svga_winsys_buffer_destroy(&vws->base, vsrf->buf);
             vsrf->buf = vbuf;
+
+            /* Rebind persistent maps immediately */
+            if (flags & PIPE_TRANSFER_PERSISTENT) {
+               *rebind = TRUE;
+               vsrf->rebind = FALSE;
+            }
             goto out_mapped;
          } else
             vmw_svga_winsys_buffer_destroy(&vws->base, vbuf);
@@ -154,7 +156,7 @@ out_mapped:
    vsrf->data = data;
    vsrf->map_mode = flags & (PIPE_TRANSFER_READ | PIPE_TRANSFER_WRITE);
 out_unlock:
-   pipe_mutex_unlock(vsrf->mutex);
+   mtx_unlock(&vsrf->mutex);
    return data;
 }
 
@@ -165,15 +167,15 @@ vmw_svga_winsys_surface_unmap(struct svga_winsys_context *swc,
                               boolean *rebind)
 {
    struct vmw_svga_winsys_surface *vsrf = vmw_svga_winsys_surface(srf);
-   pipe_mutex_lock(vsrf->mutex);
+   mtx_lock(&vsrf->mutex);
    if (--vsrf->mapcount == 0) {
       *rebind = vsrf->rebind;
       vsrf->rebind = FALSE;
-      vmw_svga_winsys_buffer_unmap(&vsrf->screen->base, vsrf->buf);
    } else {
       *rebind = FALSE;
    }
-   pipe_mutex_unlock(vsrf->mutex);
+   vmw_svga_winsys_buffer_unmap(&vsrf->screen->base, vsrf->buf);
+   mtx_unlock(&vsrf->mutex);
 }
 
 void
@@ -201,7 +203,7 @@ vmw_svga_winsys_surface_reference(struct vmw_svga_winsys_surface **pdst,
       assert(p_atomic_read(&dst->validated) == 0);
       dst->sid = SVGA3D_INVALID_ID;
 #endif
-      pipe_mutex_destroy(dst->mutex);
+      mtx_destroy(&dst->mutex);
       FREE(dst);
    }
 
