@@ -26,17 +26,16 @@
  *
  **************************************************************************/
 
-#if !defined(ANDROID) || ANDROID_API_LEVEL >= 26
-/* Android's libc began supporting shm in Oreo */
-#define HAVE_SHM
+#ifdef HAVE_SYS_SHM_H
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #endif
 
 #include "pipe/p_compiler.h"
 #include "pipe/p_format.h"
+#include "pipe/p_state.h"
 #include "util/u_inlines.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
 
@@ -78,22 +77,23 @@ dri_sw_winsys( struct sw_winsys *ws )
 }
 
 
-static boolean
+static bool
 dri_sw_is_displaytarget_format_supported( struct sw_winsys *ws,
                                           unsigned tex_usage,
                                           enum pipe_format format )
 {
    /* TODO: check visuals or other sensible thing here */
-   return TRUE;
+   return true;
 }
 
-#ifdef HAVE_SHM
+#ifdef HAVE_SYS_SHM_H
 static char *
 alloc_shm(struct dri_sw_displaytarget *dri_sw_dt, unsigned size)
 {
    char *addr;
 
-   dri_sw_dt->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT|0777);
+   /* 0600 = user read+write */
+   dri_sw_dt->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT | 0600);
    if (dri_sw_dt->shmid < 0)
       return NULL;
 
@@ -138,7 +138,7 @@ dri_sw_displaytarget_create(struct sw_winsys *winsys,
 
    dri_sw_dt->shmid = -1;
 
-#ifdef HAVE_SHM
+#ifdef HAVE_SYS_SHM_H
    if (ws->lf->put_image_shm)
       dri_sw_dt->data = alloc_shm(dri_sw_dt, size);
 #endif
@@ -165,7 +165,7 @@ dri_sw_displaytarget_destroy(struct sw_winsys *ws,
    struct dri_sw_displaytarget *dri_sw_dt = dri_sw_displaytarget(dt);
 
    if (dri_sw_dt->shmid >= 0) {
-#ifdef HAVE_SHM
+#ifdef HAVE_SYS_SHM_H
       shmdt(dri_sw_dt->data);
       shmctl(dri_sw_dt->shmid, IPC_RMID, 0);
 #endif
@@ -215,7 +215,7 @@ dri_sw_displaytarget_from_handle(struct sw_winsys *winsys,
    return NULL;
 }
 
-static boolean
+static bool
 dri_sw_displaytarget_get_handle(struct sw_winsys *winsys,
                                 struct sw_displaytarget *dt,
                                 struct winsys_handle *whandle)
@@ -224,12 +224,12 @@ dri_sw_displaytarget_get_handle(struct sw_winsys *winsys,
 
    if (whandle->type == WINSYS_HANDLE_TYPE_SHMID) {
       if (dri_sw_dt->shmid < 0)
-         return FALSE;
+         return false;
       whandle->handle = dri_sw_dt->shmid;
-      return TRUE;
+      return true;
    }
 
-   return FALSE;
+   return false;
 }
 
 static void
