@@ -37,8 +37,7 @@ public:
                    struct brw_vs_prog_data *vs_prog_data,
                    const nir_shader *shader,
                    void *mem_ctx,
-                   int shader_time_index,
-                   bool use_legacy_snorm_formula);
+                   int shader_time_index);
 
 protected:
    virtual void setup_payload();
@@ -50,13 +49,9 @@ protected:
 
 private:
    int setup_attributes(int payload_reg);
-   void setup_uniform_clipplane_values();
-   void emit_clip_distances(dst_reg reg, int offset);
 
    const struct brw_vs_prog_key *const key;
    struct brw_vs_prog_data * const vs_prog_data;
-
-   bool use_legacy_snorm_formula;
 };
 
 } /* namespace brw */
