@@ -38,7 +38,7 @@
 #include "brw_state.h"
 #include "intel_buffer_objects.h"
 #include "intel_fbo.h"
-#include "common/gen_debug.h"
+#include "dev/gen_debug.h"
 
 #define FILE_DEBUG_FLAG DEBUG_BLORP
 
@@ -125,8 +125,7 @@ blorp_surf_for_miptree(struct brw_context *brw,
                        enum isl_aux_usage aux_usage,
                        bool is_render_target,
                        unsigned *level,
-                       unsigned start_layer, unsigned num_layers,
-                       struct isl_surf tmp_surfs[1])
+                       unsigned start_layer, unsigned num_layers)
 {
    const struct gen_device_info *devinfo = &brw->screen->devinfo;
 
@@ -300,10 +299,16 @@ brw_blorp_blit_miptrees(struct brw_context *brw,
        dst_level, dst_layer, dst_x0, dst_y0, dst_x1, dst_y1,
        mirror_x, mirror_y);
 
-   if (!decode_srgb && _mesa_get_format_color_encoding(src_format) == GL_SRGB)
+   if (src_format == MESA_FORMAT_NONE)
+      src_format = src_mt->format;
+
+   if (dst_format == MESA_FORMAT_NONE)
+      dst_format = dst_mt->format;
+
+   if (!decode_srgb)
       src_format = _mesa_get_srgb_format_linear(src_format);
 
-   if (!encode_srgb && _mesa_get_format_color_encoding(dst_format) == GL_SRGB)
+   if (!encode_srgb)
       dst_format = _mesa_get_srgb_format_linear(dst_format);
 
    /* When doing a multisample resolve of a GL_LUMINANCE32F or GL_INTENSITY32F
@@ -390,7 +395,7 @@ brw_blorp_blit_miptrees(struct brw_context *brw,
    /* We do format workarounds for some depth formats so we can't reliably
     * sample with HiZ.  One of these days, we should fix that.
     */
-   if (src_aux_usage == ISL_AUX_USAGE_HIZ)
+   if (src_aux_usage == ISL_AUX_USAGE_HIZ && src_mt->format != src_format)
       src_aux_usage = ISL_AUX_USAGE_NONE;
    const bool src_clear_supported =
       src_aux_usage != ISL_AUX_USAGE_NONE && src_mt->format == src_format;
@@ -406,12 +411,11 @@ brw_blorp_blit_miptrees(struct brw_context *brw,
    intel_miptree_prepare_access(brw, dst_mt, dst_level, 1, dst_layer, 1,
                                 dst_aux_usage, dst_clear_supported);
 
-   struct isl_surf tmp_surfs[2];
    struct blorp_surf src_surf, dst_surf;
    blorp_surf_for_miptree(brw, &src_surf, src_mt, src_aux_usage, false,
-                          &src_level, src_layer, 1, &tmp_surfs[0]);
+                          &src_level, src_layer, 1);
    blorp_surf_for_miptree(brw, &dst_surf, dst_mt, dst_aux_usage, true,
-                          &dst_level, dst_layer, 1, &tmp_surfs[1]);
+                          &dst_level, dst_layer, 1);
 
    struct isl_swizzle src_isl_swizzle = {
       .r = swizzle_to_scs(GET_SWZ(src_swizzle, 0)),
@@ -459,6 +463,15 @@ brw_blorp_copy_miptrees(struct brw_context *brw,
    bool src_clear_supported, dst_clear_supported;
 
    switch (src_mt->aux_usage) {
+   case ISL_AUX_USAGE_HIZ:
+      if (intel_miptree_sample_with_hiz(brw, src_mt)) {
+         src_aux_usage = src_mt->aux_usage;
+         src_clear_supported = true;
+      } else {
+         src_aux_usage = ISL_AUX_USAGE_NONE;
+         src_clear_supported = false;
+      }
+      break;
    case ISL_AUX_USAGE_MCS:
    case ISL_AUX_USAGE_CCS_E:
       src_aux_usage = src_mt->aux_usage;
@@ -497,12 +510,11 @@ brw_blorp_copy_miptrees(struct brw_context *brw,
    intel_miptree_prepare_access(brw, dst_mt, dst_level, 1, dst_layer, 1,
                                 dst_aux_usage, dst_clear_supported);
 
-   struct isl_surf tmp_surfs[2];
    struct blorp_surf src_surf, dst_surf;
    blorp_surf_for_miptree(brw, &src_surf, src_mt, src_aux_usage, false,
-                          &src_level, src_layer, 1, &tmp_surfs[0]);
+                          &src_level, src_layer, 1);
    blorp_surf_for_miptree(brw, &dst_surf, dst_mt, dst_aux_usage, true,
-                          &dst_level, dst_layer, 1, &tmp_surfs[1]);
+                          &dst_level, dst_layer, 1);
 
    /* The hardware seems to have issues with having a two different format
     * views of the same texture in the sampler cache at the same time.  It's
@@ -931,16 +943,6 @@ blorp_get_client_format(struct brw_context *brw,
    return _mesa_tex_format_from_format_and_type(&brw->ctx, format, type);
 }
 
-static bool
-need_signed_unsigned_int_conversion(mesa_format src_format,
-                                    mesa_format dst_format)
-{
-   const GLenum src_type = _mesa_get_format_datatype(src_format);
-   const GLenum dst_type = _mesa_get_format_datatype(dst_format);
-   return (src_type == GL_INT && dst_type == GL_UNSIGNED_INT) ||
-          (src_type == GL_UNSIGNED_INT && dst_type == GL_INT);
-}
-
 bool
 brw_blorp_upload_miptree(struct brw_context *brw,
                          struct intel_mipmap_tree *dst_mt,
@@ -962,13 +964,6 @@ brw_blorp_upload_miptree(struct brw_context *brw,
       return false;
    }
 
-   /* This function relies on blorp_blit to upload the pixel data to the
-    * miptree.  But, blorp_blit doesn't support signed to unsigned or
-    * unsigned to signed integer conversions.
-    */
-   if (need_signed_unsigned_int_conversion(src_format, dst_format))
-      return false;
-
    uint32_t src_offset, src_row_stride, src_image_stride;
    struct brw_bo *src_bo =
       blorp_get_client_bo(brw, width, height, depth,
@@ -1062,13 +1057,6 @@ brw_blorp_download_miptree(struct brw_context *brw,
       return false;
    }
 
-   /* This function relies on blorp_blit to download the pixel data from the
-    * miptree. But, blorp_blit doesn't support signed to unsigned or unsigned
-    * to signed integer conversions.
-    */
-   if (need_signed_unsigned_int_conversion(src_format, dst_format))
-      return false;
-
    /* We can't fetch from LUMINANCE or intensity as that would require a
     * non-trivial swizzle.
     */
@@ -1218,7 +1206,7 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
    uint32_t x0, x1, y0, y1;
 
    mesa_format format = irb->Base.Base.Format;
-   if (!encode_srgb && _mesa_get_format_color_encoding(format) == GL_SRGB)
+   if (!encode_srgb)
       format = _mesa_get_srgb_format_linear(format);
    enum isl_format isl_format = brw->mesa_to_isl_render_format[format];
 
@@ -1238,6 +1226,9 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
 
    bool can_fast_clear = !partial_clear;
 
+   if (INTEL_DEBUG & DEBUG_NO_FAST_CLEAR)
+      can_fast_clear = false;
+
    bool color_write_disable[4] = { false, false, false, false };
    if (set_write_disables(irb, GET_COLORMASK(ctx->Color.ColorMask, buf),
                           color_write_disable))
@@ -1276,11 +1267,6 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
       }
    }
 
-   /* FINISHME: Debug and enable fast clears */
-   const struct gen_device_info *devinfo = &brw->screen->devinfo;
-   if (devinfo->gen >= 11)
-      can_fast_clear = false;
-
    if (can_fast_clear) {
       const enum isl_aux_state aux_state =
          intel_miptree_get_aux_state(irb->mt, irb->mt_level, irb->mt_layer);
@@ -1288,22 +1274,21 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
          brw_meta_convert_fast_clear_color(brw, irb->mt,
                                            &ctx->Color.ClearColor);
 
-      intel_miptree_set_clear_color(brw, irb->mt, clear_color);
-
-      /* If the buffer is already in ISL_AUX_STATE_CLEAR, the clear
-       * is redundant and can be skipped.
+      /* If the buffer is already in ISL_AUX_STATE_CLEAR and the clear color
+       * hasn't changed, the clear is redundant and can be skipped.
        */
-      if (aux_state == ISL_AUX_STATE_CLEAR)
+      if (!intel_miptree_set_clear_color(brw, irb->mt, clear_color) &&
+          aux_state == ISL_AUX_STATE_CLEAR) {
          return;
+      }
 
       DBG("%s (fast) to mt %p level %d layers %d+%d\n", __FUNCTION__,
           irb->mt, irb->mt_level, irb->mt_layer, num_layers);
 
       /* We can't setup the blorp_surf until we've allocated the MCS above */
-      struct isl_surf isl_tmp[2];
       struct blorp_surf surf;
       blorp_surf_for_miptree(brw, &surf, irb->mt, irb->mt->aux_usage, true,
-                             &level, irb->mt_layer, num_layers, isl_tmp);
+                             &level, irb->mt_layer, num_layers);
 
       /* Ivybrigde PRM Vol 2, Part 1, "11.7 MCS Buffer for Render Target(s)":
        *
@@ -1320,11 +1305,10 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
       brw_emit_end_of_pipe_sync(brw, PIPE_CONTROL_RENDER_TARGET_FLUSH);
 
       struct blorp_batch batch;
-      blorp_batch_init(&brw->blorp, &batch, brw,
-                       BLORP_BATCH_NO_UPDATE_CLEAR_COLOR);
-      blorp_fast_clear(&batch, &surf, isl_format,
-                       level, irb->mt_layer, num_layers,
-                       x0, y0, x1, y1);
+      blorp_batch_init(&brw->blorp, &batch, brw, 0);
+      blorp_fast_clear(&batch, &surf, isl_format_srgb_to_linear(isl_format),
+                       ISL_SWIZZLE_IDENTITY,
+                       level, irb->mt_layer, num_layers, x0, y0, x1, y1);
       blorp_batch_finish(&batch);
 
       brw_emit_end_of_pipe_sync(brw, PIPE_CONTROL_RENDER_TARGET_FLUSH);
@@ -1346,10 +1330,9 @@ do_single_blorp_clear(struct brw_context *brw, struct gl_framebuffer *fb,
       intel_miptree_prepare_render(brw, irb->mt, level, irb->mt_layer,
                                    num_layers, aux_usage);
 
-      struct isl_surf isl_tmp[2];
       struct blorp_surf surf;
       blorp_surf_for_miptree(brw, &surf, irb->mt, aux_usage, true,
-                             &level, irb->mt_layer, num_layers, isl_tmp);
+                             &level, irb->mt_layer, num_layers);
 
       union isl_color_value clear_color;
       memcpy(clear_color.f32, ctx->Color.ClearColor.f, sizeof(float) * 4);
@@ -1441,8 +1424,7 @@ brw_blorp_clear_depth_stencil(struct brw_context *brw,
    if (x0 == x1 || y0 == y1)
       return;
 
-   uint32_t level, start_layer, num_layers;
-   struct isl_surf isl_tmp[4];
+   uint32_t level = 0, start_layer = 0, num_layers;
    struct blorp_surf depth_surf, stencil_surf;
 
    struct intel_mipmap_tree *depth_mt = NULL;
@@ -1459,8 +1441,7 @@ brw_blorp_clear_depth_stencil(struct brw_context *brw,
 
       unsigned depth_level = level;
       blorp_surf_for_miptree(brw, &depth_surf, depth_mt, depth_mt->aux_usage,
-                             true, &depth_level, start_layer, num_layers,
-                             &isl_tmp[0]);
+                             true, &depth_level, start_layer, num_layers);
       assert(depth_level == level);
    }
 
@@ -1489,8 +1470,7 @@ brw_blorp_clear_depth_stencil(struct brw_context *brw,
       unsigned stencil_level = level;
       blorp_surf_for_miptree(brw, &stencil_surf, stencil_mt,
                              ISL_AUX_USAGE_NONE, true,
-                             &stencil_level, start_layer, num_layers,
-                             &isl_tmp[2]);
+                             &stencil_level, start_layer, num_layers);
    }
 
    assert((mask & BUFFER_BIT_DEPTH) || stencil_mask);
@@ -1525,11 +1505,9 @@ brw_blorp_resolve_color(struct brw_context *brw, struct intel_mipmap_tree *mt,
 
    const mesa_format format = _mesa_get_srgb_format_linear(mt->format);
 
-   struct isl_surf isl_tmp[1];
    struct blorp_surf surf;
    blorp_surf_for_miptree(brw, &surf, mt, mt->aux_usage, true,
-                          &level, layer, 1 /* num_layers */,
-                          isl_tmp);
+                          &level, layer, 1 /* num_layers */);
 
    /* Ivybrigde PRM Vol 2, Part 1, "11.7 MCS Buffer for Render Target(s)":
     *
@@ -1570,11 +1548,10 @@ brw_blorp_mcs_partial_resolve(struct brw_context *brw,
    const mesa_format format = _mesa_get_srgb_format_linear(mt->format);
    enum isl_format isl_format = brw_blorp_to_isl_format(brw, format, true);
 
-   struct isl_surf isl_tmp[1];
    struct blorp_surf surf;
    uint32_t level = 0;
    blorp_surf_for_miptree(brw, &surf, mt, ISL_AUX_USAGE_MCS, true,
-                          &level, start_layer, num_layers, isl_tmp);
+                          &level, start_layer, num_layers);
 
    struct blorp_batch batch;
    blorp_batch_init(&brw->blorp, &batch, brw, 0);
@@ -1667,10 +1644,9 @@ intel_hiz_exec(struct brw_context *brw, struct intel_mipmap_tree *mt,
 
    assert(mt->aux_usage == ISL_AUX_USAGE_HIZ && mt->aux_buf);
 
-   struct isl_surf isl_tmp[2];
    struct blorp_surf surf;
    blorp_surf_for_miptree(brw, &surf, mt, ISL_AUX_USAGE_HIZ, true,
-                          &level, start_layer, num_layers, isl_tmp);
+                          &level, start_layer, num_layers);
 
    struct blorp_batch batch;
    blorp_batch_init(&brw->blorp, &batch, brw,
