@@ -28,24 +28,6 @@
 
 #include "brw_context.h"
 
-bool
-intelEmitCopyBlit(struct brw_context *brw,
-                  GLuint cpp,
-                  GLshort src_pitch,
-                  drm_intel_bo *src_buffer,
-                  GLuint src_offset,
-                  uint32_t src_tiling,
-                  uint32_t src_tr_mode,
-                  GLshort dst_pitch,
-                  drm_intel_bo *dst_buffer,
-                  GLuint dst_offset,
-                  uint32_t dst_tiling,
-                  uint32_t dst_tr_mode,
-                  GLshort srcx, GLshort srcy,
-                  GLshort dstx, GLshort dsty,
-                  GLshort w, GLshort h,
-                  GLenum logicop);
-
 bool intel_miptree_blit_compatible_formats(mesa_format src, mesa_format dst);
 
 bool intel_miptree_blit(struct brw_context *brw,
@@ -56,7 +38,16 @@ bool intel_miptree_blit(struct brw_context *brw,
                         int dst_level, int dst_slice,
                         uint32_t dst_x, uint32_t dst_y, bool dst_flip,
                         uint32_t width, uint32_t height,
-                        GLenum logicop);
+                        enum gl_logicop_mode logicop);
+
+bool intel_miptree_copy(struct brw_context *brw,
+                        struct intel_mipmap_tree *src_mt,
+                        int src_level, int src_slice,
+                        uint32_t src_x, uint32_t src_y,
+                        struct intel_mipmap_tree *dst_mt,
+                        int dst_level, int dst_slice,
+                        uint32_t dst_x, uint32_t dst_y,
+                        uint32_t src_width, uint32_t src_height);
 
 bool
 intelEmitImmediateColorExpandBlit(struct brw_context *brw,
@@ -64,17 +55,11 @@ intelEmitImmediateColorExpandBlit(struct brw_context *brw,
 				  GLubyte *src_bits, GLuint src_size,
 				  GLuint fg_color,
 				  GLshort dst_pitch,
-				  drm_intel_bo *dst_buffer,
+				  struct brw_bo *dst_buffer,
 				  GLuint dst_offset,
-				  uint32_t dst_tiling,
+                                  enum isl_tiling dst_tiling,
 				  GLshort x, GLshort y,
 				  GLshort w, GLshort h,
-				  GLenum logic_op);
-void intel_emit_linear_blit(struct brw_context *brw,
-			    drm_intel_bo *dst_bo,
-			    unsigned int dst_offset,
-			    drm_intel_bo *src_bo,
-			    unsigned int src_offset,
-			    unsigned int size);
+				  enum gl_logicop_mode logic_op);
 
 #endif
