@@ -74,30 +74,38 @@ _mesa_shader_debug(struct gl_context *ctx, GLenum type, GLuint *id,
                    const char *msg);
 
 extern void
-_mesa_gl_vdebug(struct gl_context *ctx,
+_mesa_gl_vdebugf(struct gl_context *ctx,
+                 GLuint *id,
+                 enum mesa_debug_source source,
+                 enum mesa_debug_type type,
+                 enum mesa_debug_severity severity,
+                 const char *fmtString,
+                 va_list args);
+
+extern void
+_mesa_gl_debugf(struct gl_context *ctx,
                 GLuint *id,
                 enum mesa_debug_source source,
                 enum mesa_debug_type type,
                 enum mesa_debug_severity severity,
-                const char *fmtString,
-                va_list args);
+                const char *fmtString, ...) PRINTFLIKE(6, 7);
 
-extern void
+extern size_t
 _mesa_gl_debug(struct gl_context *ctx,
                GLuint *id,
                enum mesa_debug_source source,
                enum mesa_debug_type type,
                enum mesa_debug_severity severity,
-               const char *fmtString, ...) PRINTFLIKE(6, 7);
+               const char *msg);
 
 #define _mesa_perf_debug(ctx, sev, ...) do {                              \
    static GLuint msg_id = 0;                                              \
    if (unlikely(ctx->Const.ContextFlags & GL_CONTEXT_FLAG_DEBUG_BIT)) {   \
-      _mesa_gl_debug(ctx, &msg_id,                                        \
-                     MESA_DEBUG_SOURCE_API,                               \
-                     MESA_DEBUG_TYPE_PERFORMANCE,                         \
-                     sev,                                                 \
-                     __VA_ARGS__);                                        \
+      _mesa_gl_debugf(ctx, &msg_id,                                       \
+                      MESA_DEBUG_SOURCE_API,                              \
+                      MESA_DEBUG_TYPE_PERFORMANCE,                        \
+                      sev,                                                \
+                      __VA_ARGS__);                                       \
    }                                                                      \
 } while (0)
 
