@@ -329,7 +329,7 @@ put_s8_values(struct gl_context *ctx, struct gl_renderbuffer *rb,
               const GLubyte stencil[])
 {
    const GLint w = rb->Width, h = rb->Height;
-   gl_pack_ubyte_stencil_func pack_stencil =
+   mesa_pack_ubyte_stencil_func pack_stencil =
       _mesa_get_pack_ubyte_stencil_func(rb->Format);
    GLuint i;
 
