@@ -2,6 +2,7 @@
  * Mesa 3-D graphics library
  *
  * Copyright (C) 1999-2007  Brian Paul   All Rights Reserved.
+ * Copyright (C) 2018-2019 Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -40,6 +41,20 @@ float _mesa_half_to_float(uint16_t val);
 uint8_t _mesa_half_to_unorm8(uint16_t v);
 uint16_t _mesa_uint16_div_64k_to_half(uint16_t v);
 
+/*
+ * _mesa_float_to_float16_rtz is no more than a wrapper to the counterpart
+ * softfloat.h call. Still, softfloat.h conversion API is meant to be kept
+ * private. In other words, only use the API published here, instead of
+ * calling directly the softfloat.h one.
+ */
+uint16_t _mesa_float_to_float16_rtz(float val);
+
+static inline uint16_t
+_mesa_float_to_float16_rtne(float val)
+{
+   return _mesa_float_to_half(val);
+}
+
 static inline bool
 _mesa_half_is_negative(uint16_t h)
 {
